
"""
.. module:: FD_operators_equations.py
    :synopsis: Contains functions that implement finite-difference forms of
     mathematical operators, and functions that solve equations.
.. moduleauthor:: Kereman Alonso (alonso.kerman5@gmail.com)
"""

import numpy as np
from numpy import float64
import numba


@numba.jit(nopython=True)
def centered_partial_vertical(F, d, boundary, old_derivative):
    """
        This function takes a 2D matrix that contains the values of a function
        and returns a matrix that contains the partial derivatives of the
        function calculated along columns. Partial derivative is computed using
        centered differencing.

    """

    derivative = np.zeros(F.shape, dtype=float64)
    nx = F.shape[0]
    ny = F.shape[1]
    # Cyclic boundary conditions
    for i in range(nx):
        for j in range(ny):
            derivative[i, j] = (F[(i+1) % nx, j] - F[i-1, j]) / (2 * d)
    # Fixed boundary conditions
    if (boundary == 'fixed'):
        derivative[0, :] = old_derivative[0, :]
        derivative[-1, :] = old_derivative[-1, :]
        derivative[:, 0] = old_derivative[:, 0]
        derivative[:, -1] = old_derivative[:, -1]
    # Mixed boundary conditions
    if (boundary == 'mixed'):
        derivative[0, :] = (F[1, :] - F[0, :]) / d
        derivative[nx-1, :] = (F[nx-1, :] - F[nx-2, :]) / d
    return derivative


@numba.jit(nopython=True)
def centered_partial_horizontal(F, d, boundary, old_derivative):
    """
        This function takes a 2D matrix that contains the values of a function
        and returns a matrix that contains the partial derivatives of the
        function calculated along rows. Partial derivative is computed using
        centered differencing.

    """

    derivative = np.zeros(F.shape, dtype=float64)
    nx = F.shape[0]
    ny = F.shape[1]
    # Cyclic boundary conditions
    for i in range(nx):
        for j in range(ny):
            derivative[i, j] = (F[i, (j+1) % ny] - F[i, j-1]) / (2 * d)
    # Fixed boundary conditions
    if (boundary == 'fixed'):
        derivative[0, :] = old_derivative[0, :]
        derivative[-1, :] = old_derivative[-1, :]
        derivative[:, 0] = old_derivative[:, 0]
        derivative[:, -1] = old_derivative[:, -1]
    return derivative


@numba.jit(nopython=True)
def laplacian_5(psi, d, boundary, old_laplacian):
    """
        This function takes a 2D matrix and returns its Laplacian. The
        aproximation uses information at five grid points.

    """
    laplacian_matrix = np.zeros(psi.shape, dtype=float64)
    nx = psi.shape[0]
    ny = psi.shape[1]

    # Cyclic boundary conditions
    for i in range(nx):
        for j in range(ny):
            laplacian_matrix[i, j] = 1.0 / (d ** 2) *\
                                     (psi[(i+1) % nx, j] + psi[i-1, j]
                                      + psi[i, (j+1) % ny] + psi[i, j-1]
                                      - 4 * psi[i, j])

    # Fixed boundary conditions
    if (boundary == 'fixed'):
        laplacian_matrix[0, :] = old_laplacian[0, :]
        laplacian_matrix[-1, :] = old_laplacian[-1, :]
        laplacian_matrix[:, 0] = old_laplacian[:, 0]
        laplacian_matrix[:, -1] = old_laplacian[:, -1]

    return laplacian_matrix


@numba.jit(nopython=True)
def laplacian_9(psi, d, boundary, old_laplacian):
    """
        This function takes a 2D matrix and returns its Laplacian. The
        aproximation uses information at nine grid points.

    """
    laplacian_matrix = np.zeros(psi.shape, dtype=float64)
    nx = psi.shape[0]
    ny = psi.shape[1]

    # Cyclic boundary conditions
    for i in range(nx):
        for j in range(ny):
            laplacian_matrix[i, j] = 1.0 / (6 * d ** 2) *\
                                     (4 * (psi[(i+1) % nx, j]
                                     + psi[i-1, j] + psi[i, (j+1) % ny]
                                     + psi[i, j-1])
                                     + psi[(i+1) % nx, (j+1) % ny]
                                     + psi[(i+1) % nx, j-1] + psi[i-1, j-1]
                                     + psi[i-1, (j+1) % ny] - 20 * psi[i, j])

    # Fixed boundary conditions
    if (boundary == 'fixed'):
        laplacian_matrix[0, :] = old_laplacian[0, :]
        laplacian_matrix[-1, :] = old_laplacian[-1, :]
        laplacian_matrix[:, 0] = old_laplacian[:, 0]
        laplacian_matrix[:, -1] = old_laplacian[:, -1]

    return laplacian_matrix


@numba.jit(nopython=True)
def jacobian(psi, zeta, d, boundary, old_jacobian):
    """
        This function takes psi and zeta 2D matrices and returns the
        Jacobian.

    """

    jacobian_matrix = np.zeros(psi.shape, dtype=float64)
    nx = psi.shape[0]
    ny = psi.shape[1]

    # Cyclic boundary conditions
    for i in range(nx):
        for j in range(ny):
            J_1 = 1.0 / (4 * d ** 2) * ((psi[(i+1) % nx, j] - psi[i-1, j]) *
                  (zeta[i, (j+1) % ny] - zeta[i, j-1]) - (psi[i, (j+1) % ny]
                  - psi[i, j-1]) * (zeta[(i+1) % nx, j] - zeta[i-1, j]))
            J_2 = 1.0 / (4 * d ** 2) * (psi[(i+1) % nx, j] * 
                  (zeta[(i+1) % nx, (j+1) % ny] - zeta[(i+1) % nx, j-1])
                  - psi[i-1, j] * (zeta[i-1, (j+1) % ny] -
                  zeta[i-1, j-1]) - psi[i, (j+1) % ny] *
                  (zeta[(i+1) % nx, (j+1) % ny] - zeta[i-1, (j+1) % ny])
                  + psi[i, j-1] * (zeta[(i+1) % nx, j-1] -
                  zeta[i-1, j-1]))
            J_3 = 1.0 / (4 * d ** 2) * (zeta[i, (j+1) % ny] *
                  (psi[(i+1) % nx, (j+1) % ny] - psi[i-1, (j+1) % ny])
                  - zeta[i, j-1] * (psi[(i+1) % nx, j-1] -
                  psi[i-1, j-1]) - zeta[(i+1) % nx, j] *
                  (psi[(i+1) % nx, (j+1) % ny] - psi[(i+1) % nx, j-1])
                  + zeta[i-1, j] * (psi[i-1, (j+1) % ny] -
                  psi[i-1, j-1]))
            jacobian_matrix[i, j] = (J_1 + J_2 + J_3) / 3

    # Fixed boundary conditions
    if (boundary == 'fixed'):
        jacobian_matrix[0:2, :] = old_jacobian[0:2, :]
        jacobian_matrix[-2:nx, :] = old_jacobian[-2:nx, :]
        jacobian_matrix[:, 0:2] = old_jacobian[:, 0:2]
        jacobian_matrix[:, -2:ny] = old_jacobian[:, -2:ny]

    return jacobian_matrix


@numba.jit(nopython=True)
def poisson_relaxation(zeta, d, threshold, boundary, old_psi):
    """
        This function takes zeta 2D matrix, wich is equal to Laplacian(psi),
        and solves the Poisson equation to get G, an approximation of
        psi matrix.

    """
    nx = old_psi.shape[0]
    ny = old_psi.shape[1]

    B = d ** 2 / 4

    # Frist guess
    G = np.zeros(old_psi.shape, dtype=float64)
    G[:] = old_psi[:]
    
    if (boundary == 'fixed'):
        # Calculate residual
        R = np.zeros(old_psi.shape, dtype=float64)
        R_max = 0.0
        laplacian_matrix_G = laplacian_5(G, d, boundary='fixed',
                                         old_laplacian=zeta)
        
        for i in range(1, nx-1):
            for j in range(1, ny-1):
                R[i, j] = laplacian_matrix_G[i, j] - zeta[i, j]
                if (R[i, j] > R_max):
                    R_max = R[i, j]
        while (R_max > threshold):
            # Update guess and recalculate residual
            R_max = 0.0
            for i in range(1, nx-1):
                for j in range(1, ny-1):
                    # Update guess
                    G[i, j] = G[i, j] + B * R[i, j]
                    # Calculate lapacian
                    laplacian_G = 1.0 / (d ** 2) *\
                                         (G[i+1, j] + G[i-1, j]
                                          + G[i, j+1] + G[i, j-1]
                                          - 4 * G[i, j])
                    # Recalculate residual
                    R[i, j] = laplacian_G - zeta[i, j]
                    if (R[i, j] > R_max):
                        R_max = R[i, j]
    if (boundary == 'cyclic'):
        # Calculate residual
        R = np.zeros(old_psi.shape, dtype=float64)
        R_max = 0.0
        laplacian_matrix_G = laplacian_5(G, d, boundary='cyclic',
                                         old_laplacian=zeta)

        for i in range(0, nx):
            for j in range(0, ny):
                R[i, j] = laplacian_matrix_G[i, j] - zeta[i, j]
                if (R[i, j] > R_max):
                    R_max = R[i, j]
        while (R_max > threshold):
            # Update guess and recalculate residual
            R_max = 0.0
            for i in range(0, nx):
                for j in range(0, ny):
                    # Update guess
                    G[i, j] = G[i, j] + B * R[i, j]
                    # Calculate lapacian
                    laplacian_G = 1.0 / (d ** 2) *\
                                         (G[(i+1) % nx, j] + G[i-1, j]
                                          + G[i, (j+1) % ny] + G[i, j-1]
                                          - 4 * G[i, j])
                    # Recalculate residual
                    R[i, j] = laplacian_G - zeta[i, j]
                    if (R[i, j] > R_max):
                        R_max = R[i, j]       
    return G


def poisson_matrix(zeta, d, boundary, old_psi):
    """
        This function takes zeta 2D matrix, wich is equal to Laplacian(psi),
        and solves the Poisson equation to get the psi matrix.

    """
    Nx = zeta.shape[0]
    Ny = zeta.shape[1]
    A = np.zeros((Nx*Ny, Nx*Ny), dtype=float64)
    tridiag = np.zeros((Ny, Ny), dtype=float64)
    for j in range(0, Ny):
            tridiag[j, j] = -4
            tridiag[j, (j+1) % Ny] = 1
            tridiag[j, j-1] = 1
    identity = np.eye(Ny, dtype=float64)

    if (boundary == 'cyclic'):
        B = d ** 2 * zeta.reshape(Nx*Ny)

        for i in range(1, Nx - 1):
            A[i*Ny:(i+1)*Ny, i*Ny:(i+1)*Ny] = tridiag[:, :]
            A[i*Ny:(i+1)*Ny, (i+1)*Ny:(i+2)*Ny] = identity[:, :]
            A[i*Ny:(i+1)*Ny, (i-1)*Ny:i*Ny] = identity[:, :]
        
        # i = 0
        A[0:Ny, 0:Ny] = tridiag[:, :]
        A[0:Ny, Ny:2*Ny] = identity[:, :]
        A[0:Ny, (Nx-1)*Ny:] = identity[:, :]
    
        # i = Nx - 1
        A[(Nx-1)*Ny:, (Nx-1)*Ny:] = tridiag[:, :]
        A[(Nx-1)*Ny:, 0:Ny] = identity[:, :]
        A[(Nx-1)*Ny:, (Nx-2)*Ny:(Nx-1)*Ny] = identity[:, :]

        A_inv = np.linalg.pinv(A)

    elif (boundary == 'fixed'):
        B = np.zeros((Nx, Ny), dtype=float64)

        for i in range(Nx):
            for j in range(Ny):
                if i == 0 or j == 0 or i == Nx - 1 or j == Ny - 1:
                    B[i, j] = old_psi[i, j]
                else:
                    B[i, j] = d ** 2 * zeta[i, j]
        B = B.reshape(Nx*Ny)

        tridiag[0, :] = identity[0, :]
        tridiag[Ny-1, :] = identity[Ny-1, :]

        for i in range(1, Nx - 1):
            A[i*Ny:(i+1)*Ny, i*Ny:(i+1)*Ny] = tridiag[:, :]
            A[i*Ny+1:(i+1)*Ny-1, (i+1)*Ny+1:(i+2)*Ny-1] =\
                identity[1:Ny-1, 1:Ny-1]
            A[i*Ny+1:(i+1)*Ny-1, (i-1)*Ny+1:i*Ny-1] =\
                identity[1:Ny-1, 1:Ny-1]
        # i = 0
        A[0:Ny, 0:Ny] = identity[:, :]

        # i = Nx - 1
        A[(Nx-1)*Ny:, (Nx-1)*Ny:] = identity[:, :]

        A_inv = np.linalg.pinv(A)
    psi = np.matmul(A_inv, B).reshape((Nx, Ny))
    return psi


def vorticity_QG_flat(zeta_old, dt, J, beta, vg):
    zeta_new = np.zeros(zeta_old.shape, dtype=float64)
    for i in range(zeta_new.shape[0]):
        for j in range(zeta_new.shape[1]):
            zeta_new[i, j] = zeta_old[i, j] - 2 * dt * (J[i, j] + beta *
                                                        vg[i, j])
