
"""
.. module:: get_external_measurements.py
    :synopsis: It contains the functions used to extract data from the
    downloaded .nc data files.
.. moduleauthor:: Kereman Alonso (alonso.kerman5@gmail.com)
"""

from netCDF4 import Dataset
import numpy as np
from numpy import float64, arcsin
from math import pi
from scipy import interpolate as interp


def old_get_vorticity_matrix(nc, time, pressure, new_sin_latit, new_longit,
                             vorticity):
    """
        This function takes .nc file and extracts the vorticity matrix
        for a given time, pressure, and physical grid. No antialiasing is
        performed.

    """

    # The project uses sin(latitude) instead of latitudes
    # (new_sin_latit = sin(latitude))
    new_latit = arcsin(new_sin_latit)

    # Get the latitudes of the nc dataset
    nc_latit = np.array(nc.variables['latitude'][:], dtype=float64)
    # Get the longitudes of the nc dataset
    nc_longit = np.array(nc.variables['longitude'][:], dtype=float64)
    # Get the vorticity of the nc dataset
    nc_vorticity = np.array(nc.variables['vo'][time, pressure, :, :],
                            dtype=float64)

    # Modify nc_longit to suit the definition of longitudes in the project:
    # [-pi, pi) rad
    nc_longit *= pi / 180
    nc_longit -= pi

    # Modify nc_latit to suit the definition of "latitudes" in the project:
    # [pi/2, -pi/2]
    nc_latit *= pi / 180

    # Invert nc_latit and 0 axis of nc_vorticity to get the interpolation
    # function using scipy library
    nc_latit = np.flip(nc_latit, 0)
    nc_vorticity = np.flip(nc_vorticity, 0)

    # Create the function that can compute interpolated values of vorticity
    f = interp.RectBivariateSpline(nc_latit, nc_longit, nc_vorticity)

    # Evaluate vorticity at the points defined by new new_latit and new_longit
    vorticity[:] = f(new_latit, new_longit)[:]
    vorticity[:] = np.flip(vorticity, 0)[:]


def get_vorticity_matrix(nc, time, pressure, new_sin_latit, new_longit,
                         order,
                         vorticity):
    """
        This function takes .nc file and extracts the vorticity matrix
        for a given time, pressure, and physical grid. Uniform supersampling
        is aplied to the .nc file to get an antialiased vorticity matrix.

    """

    # The project uses sin(latitude) instead of latitudes
    # (new_sin_latit = sin(latitude), (-pi/2, pi/2))
    new_latit = arcsin(new_sin_latit)

    # Get the latitudes of the nc dataset
    nc_latit = np.array(nc.variables['latitude'][:], dtype=float64)
    # Get the longitudes of the nc dataset
    nc_longit = np.array(nc.variables['longitude'][:], dtype=float64)
    # Get the vorticity of the nc dataset
    nc_vorticity = np.array(nc.variables['vo'][time, pressure, :, :],
                            dtype=float64)

    # Modify nc_longit to suit the definition of longitudes in the project:
    # [-pi, pi) rad
    nc_longit *= pi / 180
    nc_longit -= pi

    # Modify nc_latit to suit the definition of "latitudes" in the project:
    # [pi/2, -pi/2]
    nc_latit *= pi / 180

    # Invert nc_latit and 0 axis of nc_vorticity to get the interpolation
    # function using scipy library
    nc_latit = np.flip(nc_latit, 0)
    nc_vorticity = np.flip(nc_vorticity, 0)

    # Create the function that can compute interpolated values of vorticity
    f = interp.RectBivariateSpline(nc_latit, nc_longit, nc_vorticity)

    # Evaluate vorticity at the points defined by new new_latit and new_longit
    # by calculating the average vorticity value of points close to the
    # point defined by new_latit and new_longit
    latit_dim = new_latit.shape[0]
    longit_dim = new_longit.shape[0]
    d_latit = 0.5 * (new_latit[1] - new_latit[0])
    d_longit = 0.5 * 2 * pi / longit_dim

    for i_latit in range(latit_dim):
        for i_longit in range(longit_dim):
            latit = new_latit[i_latit]
            longit = new_longit[i_longit]
            sub_point_num = 0
            vorticity[-i_latit+latit_dim-1, i_longit] = 0
            for i_sub_latit in range(-order, order+1):
                for i_sub_longit in range(-order, order+1):
                    vorticity[-i_latit+latit_dim-1, i_longit] +=\
                        f(latit + d_latit * i_sub_latit / order,
                          longit + d_longit * i_sub_longit / order)
                    sub_point_num += 1

            vorticity[-i_latit+latit_dim-1, i_longit] /= sub_point_num


def get_u_matrix(nc, time, pressure, new_sin_latit, new_longit,
                 order, u):
    """
        This function takes .nc file and extracts the u wind component matrix
        for a given time, pressure, and physical grid. Uniform supersampling
        is aplied to the .nc file to get an antialiased u wind coponent matrix.

    """

    # The project uses sin(latitude) instead of latitudes
    # (new_sin_latit = sin(latitude), (-pi/2, pi/2))
    new_latit = arcsin(new_sin_latit)

    # Get the latitudes of the nc dataset
    nc_latit = np.array(nc.variables['latitude'][:], dtype=float64)
    # Get the longitudes of the nc dataset
    nc_longit = np.array(nc.variables['longitude'][:], dtype=float64)
    # Get u of the nc dataset
    nc_u = np.array(nc.variables['u'][time, pressure, :, :],
                    dtype=float64)

    # Modify nc_longit to suit the definition of longitudes in the project:
    # [-pi, pi) rad
    nc_longit *= pi / 180
    nc_longit -= pi

    # Modify nc_latit to suit the definition of "latitudes" in the project:
    # [pi/2, -pi/2]
    nc_latit *= pi / 180

    # Invert nc_latit and 0 axis of nc_u to get the interpolation
    # function using scipy library
    nc_latit = np.flip(nc_latit, 0)
    nc_u = np.flip(nc_u, 0)

    # Create the function that can compute interpolated values of u
    f = interp.RectBivariateSpline(nc_latit, nc_longit, nc_u)

    # Evaluate u at the points defined by new new_latit and new_longit
    # by calculating the average u value of points close to the
    # point defined by new_latit and new_longit
    latit_dim = new_latit.shape[0]
    longit_dim = new_longit.shape[0]
    d_latit = 0.5 * (new_latit[1] - new_latit[0])
    d_longit = 0.5 * 2 * pi / longit_dim

    for i_latit in range(latit_dim):
        for i_longit in range(longit_dim):
            latit = new_latit[i_latit]
            longit = new_longit[i_longit]
            sub_point_num = 0
            u[-i_latit+latit_dim-1, i_longit] = 0
            for i_sub_latit in range(-order, order+1):
                for i_sub_longit in range(-order, order+1):
                    u[-i_latit+latit_dim-1, i_longit] +=\
                        f(latit + d_latit * i_sub_latit / order,
                          longit + d_longit * i_sub_longit / order)
                    sub_point_num += 1

            u[-i_latit+latit_dim-1, i_longit] /= sub_point_num


def get_v_matrix(nc, time, pressure, new_sin_latit, new_longit,
                 order, v):
    """
        This function takes .nc file and extracts the v wind component matrix
        for a given time, pressure, and physical grid. Uniform supersampling
        is aplied to the .nc file to get an antialiased v wind coponent matrix.

    """

    # The project uses sin(latitude) instead of latitudes
    # (new_sin_latit = sin(latitude), (-pi/2, pi/2))
    new_latit = arcsin(new_sin_latit)

    # Get the latitudes of the nc dataset
    nc_latit = np.array(nc.variables['latitude'][:], dtype=float64)
    # Get the longitudes of the nc dataset
    nc_longit = np.array(nc.variables['longitude'][:], dtype=float64)
    # Get the v of the nc dataset
    nc_v = np.array(nc.variables['v'][time, pressure, :, :],
                    dtype=float64)

    # Modify nc_longit to suit the definition of longitudes in the project:
    # [-pi, pi) rad
    nc_longit *= pi / 180
    nc_longit -= pi

    # Modify nc_latit to suit the definition of "latitudes" in the project:
    # [pi/2, -pi/2]
    nc_latit *= pi / 180

    # Invert nc_latit and 0 axis of nc_vorticity to get the interpolation
    # function using scipy library
    nc_latit = np.flip(nc_latit, 0)
    nc_v = np.flip(nc_v, 0)

    # Create the function that can compute interpolated values of v
    f = interp.RectBivariateSpline(nc_latit, nc_longit, nc_v)

    # Evaluate v at the points defined by new new_latit and new_longit
    # by calculating the average v value of points close to the
    # point defined by new_latit and new_longit
    latit_dim = new_latit.shape[0]
    longit_dim = new_longit.shape[0]
    d_latit = 0.5 * (new_latit[1] - new_latit[0])
    d_longit = 0.5 * 2 * pi / longit_dim

    for i_latit in range(latit_dim):
        for i_longit in range(longit_dim):
            latit = new_latit[i_latit]
            longit = new_longit[i_longit]
            sub_point_num = 0
            v[-i_latit+latit_dim-1, i_longit] = 0
            for i_sub_latit in range(-order, order+1):
                for i_sub_longit in range(-order, order+1):
                    v[-i_latit+latit_dim-1, i_longit] +=\
                        f(latit + d_latit * i_sub_latit / order,
                          longit + d_longit * i_sub_longit / order)
                    sub_point_num += 1

            v[-i_latit+latit_dim-1, i_longit] /= sub_point_num


if __name__ == "__main__":
    import spherical_spectral as spherical

    year = '2019'
    month = '07'
    # Get the information stored in the nc file
    an = Dataset('..\\..\\data\\an_{}_{}.nc'.format(year, month), 'r')

    nc = Dataset('..\\..\\data\\ACC_fc_2019_07.nc', 'r')

    """
    Get information about nc file
    """
    # Print information about variables of an file
    for variable in an.variables:
        print([variable, an.variables[variable].units,
              an.variables[variable].shape])

    # Print information about variables of nc file
    for variable in nc.variables:
        print([variable, nc.variables[variable].units,
              nc.variables[variable].shape])

    # # Print longitude and latitude too see how they are defined
    # print(nc.variables['latitude'][:])
    # print(nc.variables['longitude'][:])

    # # Print ime and level to see how they are defined
    # print(nc.variables['time'][:])
    # print(nc.variables['level'][:])

    """
    Test get_vorticity_matrix
    """
    # time = 8
    # print("Time: {}".format(nc.variables['time'][time]))
    # pressure = 2
    # print("Level: {}".format(nc.variables['level'][pressure]))

    # # Plot original vorticity (nc values)
    # nc_latitudes = nc.variables['latitude'][:]
    # nc_longitudes = nc.variables['longitude'][:]
    # nc_vorticity = nc.variables['vo'][time, pressure, :, :]

    # # # Color map
    # # # # Normalize vorticity values
    # nc_norm = cm.colors.Normalize(vmin=nc_vorticity.min(),
    #                               vmax=nc_vorticity.max())
    # nc_fig2 = plt.figure(4, dpi=150)
    # nc_color_plot = plt.pcolormesh(nc_longitudes, nc_latitudes,
    #                                 nc_vorticity,
    #                                 cmap=cm.hot,
    #                                 norm=nc_norm)
    # plt.show()    

    # # Get vorticity matrix
    # # # order: order of antialiasing (uniform supersampling)
    # order = 2
    # M = 63
    # longit_dim = 3 * M + 1
    # latit_dim = (3 * M + 1) // 2 if (3 * M + 1) % 2 == 0 else\
    #     (3 * M + 1) // 2 + 1
    # longitudes, latitudes, weights =\
    #     spherical.coordinates_and_weights(longit_dim, latit_dim)

    # vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
    # a_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)

    # get_vorticity_matrix(nc, time, pressure, latitudes, longitudes, order,
    #                       vorticity)
    # old_get_vorticity_matrix(nc, time, pressure, latitudes, longitudes, 
    #                          a_vorticity)

    # # Plot low res aliased vorticity matrix
    # latitudes_plot = - arcsin(latitudes)
    # norm = cm.colors.Normalize(vmin=vorticity.min()*1e5,
    #                            vmax=vorticity.max()*1e5)
    # fig = plt.figure(10, dpi=150)
    # color_plot = plt.pcolormesh(longitudes, latitudes_plot, a_vorticity*1e5,
    #                             cmap=cm.hot,
    #                             norm=norm)
    # plt.show()

    # # Plot low res antialiased vorticity matrix
    # norm = cm.colors.Normalize(vmin=vorticity.min()*1e5,
    #                            vmax=vorticity.max()*1e5)
    # fig = plt.figure(1, dpi=150)
    # color_plot = plt.pcolormesh(longitudes, latitudes_plot, vorticity*1e5,
    #                             cmap=cm.hot,
    #                             norm=norm)
    # plt.show()

    # # Plot low res antialiase vorticity matrix after forward and inverse
    # # transforms

    # legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
    # beta_matrix = np.zeros((M+2, M+2), dtype=float64)

    # sph.associated_legendre(latitudes, legendre_matrix, beta_matrix)

    # t_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
    # vo_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
    # sph.forward_transform(vorticity, weights, legendre_matrix, vo_coef)
    # sph.inverse_transform(vo_coef, legendre_matrix, longitudes, t_vorticity)

    # norm = cm.colors.Normalize(vmin=vorticity.min()*1e5,
    #                             vmax=vorticity.max()*1e5)
    # fig = plt.figure(3, dpi=150)
    # color_plot = plt.pcolormesh(longitudes, latitudes_plot, t_vorticity*1e5,
    #                             cmap=cm.hot,
    #                             norm=norm)
    # plt.show()

    """
    Test get_v_matrix
    """
    
    # year = '2019'
    # month = '07'
    # # Get the information stored in the nc file
    # nc = Dataset('..\\..\\data\\an_{}_{}.nc'.format(year, month), 'r')

    # day = 20
    # t0 = (day - 1) * 4
    # pressure = 2

    # # Plot original u (nc values)
    # nc_latitudes = nc.variables['latitude'][:]
    # nc_longitudes = nc.variables['longitude'][:]
    # nc_v = nc.variables['v'][t0, pressure, :, :]

    # # # Color map
    # # # # Normalize vorticity values
    # nc_norm = cm.colors.Normalize(vmin=nc_v.min(),
    #                               vmax=nc_v.max())
    # nc_fig2 = plt.figure(4, dpi=150)
    # nc_color_plot = plt.pcolormesh(nc_longitudes, nc_latitudes,
    #                                 nc_v,
    #                                 cmap=cm.Blues,
    #                                 norm=nc_norm)
    # plt.show()    

    # # Get vorticity matrix
    # # # order: order of antialiasing (uniform supersampling)
    # order = 2
    # M = 63
    # longit_dim = 3 * M + 1
    # latit_dim = (3 * M + 1) // 2 if (3 * M + 1) % 2 == 0 else\
    #     (3 * M + 1) // 2 + 1
    # longitudes, latitudes, weights =\
    #     spherical.coordinates_and_weights(longit_dim, latit_dim)

    # v = np.zeros((latit_dim, longit_dim), dtype=float64)

    # get_v_matrix(nc, t0, pressure, latitudes, longitudes, order,
    #              v)

    # # Plot low res antialiased vorticity matrix
    # latitudes_plot = - arcsin(latitudes)
    # fig = plt.figure(10, dpi=150)
    # color_plot = plt.pcolormesh(longitudes, latitudes_plot, v,
    #                             cmap=cm.Blues,
    #                             norm=nc_norm)
    # plt.show()

    # # Plot low res antialiase vorticity matrix after forward and inverse
    # # transforms

    # legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
    # beta_matrix = np.zeros((M+2, M+2), dtype=float64)

    # sph.associated_legendre(latitudes, legendre_matrix, beta_matrix)

    # t_v = np.zeros((latit_dim, longit_dim), dtype=float64)
    # v_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
    # sph.forward_transform(v, weights, legendre_matrix, v_coef)
    # sph.inverse_transform(v_coef, legendre_matrix, longitudes, t_v)

    # fig = plt.figure(3, dpi=150)
    # color_plot = plt.pcolormesh(longitudes, latitudes_plot, t_v,
    #                             cmap=cm.Blues,
    #                             norm=nc_norm)
    # fig.colorbar(color_plot)
    # plt.show()
