
"""
.. module:: module_3DVAR.py
    :synopsis: It contains the functions used in applying the 3DVAR technique.
.. moduleauthor:: Kereman Alonso (alonso.kerman5@gmail.com)
"""

import numpy as np
from numpy import float64

import module_basic_BVE as sph


def compute_B_24(year_months, latit_dim, longit_dim):
    """
        This function takes BVE vorticity forecasts to compute a B matrix
        appropiate for applying 3DVAR every 24h. It wasn't used in the
        final version.

    """
    flattened_dim = latit_dim * longit_dim
    B = np.zeros(flattened_dim, dtype=float64)
    tot_day_num = len(year_months) * 30
    for year_month in year_months:
        for day in range(1, 31):
            with open('spherical_csv\\B_3DVAR\\vo_{}_{}_{}_{}.csv'.format(
                    year_month, day, 0, 24),
                    "r", newline='') as csvfile:
                vo_f24 = np.loadtxt(csvfile, dtype=np.float64,
                                    delimiter=",")
                xb_f24 = vo_f24.flatten()
            with open('spherical_csv\\B_3DVAR\\vo_{}_{}_{}_{}.csv'.format(
                    year_month, day, 12, 12),
                    "r", newline='') as csvfile:
                vo_f12 = np.loadtxt(csvfile, dtype=np.float64,
                                    delimiter=",")
                xb_f12 = vo_f12.flatten()

            for vo_i in range(flattened_dim):
                sigma = (xb_f24[vo_i] - xb_f12[vo_i]) / xb_f12[vo_i]
                # sigma values greater than one are assumed to be erroneous
                # and are discarded
                if abs(sigma) > 1:
                    sigma = 0
                B[vo_i] += 1 / tot_day_num * sigma ** 2
    with open('spherical_csv\\B_3DVAR\\B_24.csv', "w", newline='') as csvfile:
        np.savetxt(csvfile, B, delimiter=",")


def compute_B_12(year_months, latit_dim, longit_dim):
    """
        This function takes BVE vorticity forecasts to compute a B matrix
        appropiate for applying 3DVAR every 12h.

    """
    flattened_dim = latit_dim * longit_dim
    B = np.zeros(flattened_dim, dtype=float64)
    tot_day_num = len(year_months) * 30
    for year_month in year_months:
        for day in range(1, 31):
            with open('spherical_csv\\B_3DVAR\\vo_{}_{}_{}_{}.csv'.format(
                    year_month, day, 12, 12),
                    "r", newline='') as csvfile:
                vo_f12 = np.loadtxt(csvfile, dtype=np.float64,
                                    delimiter=",")
                xb_f12 = vo_f12.flatten()
            with open('spherical_csv\\B_3DVAR\\vo_{}_{}_{}_{}.csv'.format(
                    year_month, day, 18, 6),
                    "r", newline='') as csvfile:
                vo_f6 = np.loadtxt(csvfile, dtype=np.float64,
                                   delimiter=",")
                xb_f6 = vo_f6.flatten()

            for vo_i in range(flattened_dim):
                sigma = xb_f12[vo_i] - xb_f6[vo_i]
                B[vo_i] += 1 / tot_day_num * sigma ** 2
    with open('spherical_csv\\B_3DVAR\\B_12.csv', "w", newline='') as csvfile:
        np.savetxt(csvfile, B, delimiter=",")


def read_B(step_3DVAR, B):
    """
        This function reads the B matrix stored in a .csv file.

    """
    with open('spherical_csv\\B_3DVAR\\B_{}.csv'.format(step_3DVAR),
              "r", newline='') as csvfile:
        B[:] = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")[:]


def get_y(vo_an, H, y):
    """
        This function computes the y vector.

    """
    vo_an_flattened = vo_an.flatten()
    y_dim = y.shape[0]

    for i in range(y_dim):
        point = H[i]
        y[i] = vo_an_flattened[point]


def get_H(ob_point_num, latit_dim, longit_dim, H):
    """
        This function computes the H matrix.

    """
    flattened_dim = latit_dim * longit_dim
    indices = np.array([i for i in range(flattened_dim)], dtype=int)
    observation_points = np.random.choice(indices, size=ob_point_num,
                                          replace=False)
    for i in range(ob_point_num):
        H[i] = observation_points[i]


def compute_R_inv(sigma, y, dif_y_x, R_inv):
    """
        This function computes the inverse of the R matrix.

    """
    for i in range(R_inv.shape[0]):
        if (dif_y_x[i] > sigma * y[i] * 5):
            R_inv[i] = 0
        elif (dif_y_x[i] > sigma * y[i] * 3):
            R_inv[i] = 1 / dif_y_x[i] ** 2
        else:
            R_inv[i] = 1 / (sigma * y[i]) ** 2


def apply_3DVAR(vo_forecast, vo_an, latit_dim, longit_dim, weights,
                vo_coef, legendre_matrix, longitudes,
                ob_point_num, H, step_3DVAR,
                vo_3DVAR):
    """
        This function takes in forecasts and analysis (that are used as
        observations) vorticity matrices and returns the result of the
        3DVAR analysis.

    """
    flattened_dim = latit_dim * longit_dim
    xb = vo_forecast.flatten()
    x = vo_forecast.flatten()

    B = np.zeros(flattened_dim, dtype=float64)
    read_B(step_3DVAR, B)

    B_inv = np.zeros(flattened_dim, dtype=float64)
    B_inv[:] = 1 / B[:]

    y = np.zeros(ob_point_num, dtype=float64)
    get_y(vo_an, H, y)

    sigma = 0.10
    R_inv = np.zeros(ob_point_num, dtype=float64)

    Hxb = np.zeros(y.shape[0], dtype=float64)
    for i in range(y.shape[0]):
        point = H[i]
        Hxb[i] = xb[point]

    Hx = np.zeros(y.shape[0], dtype=float64)

    H_R_H = np.zeros(flattened_dim, dtype=float64)
    C = np.zeros(flattened_dim, dtype=float64)

    for i in range(y.shape[0]):
        point = H[i]
        Hx[i] = x[point]

    dif_y_x = y - Hx
    compute_R_inv(sigma, y, dif_y_x, R_inv)
    A = np.multiply(R_inv, dif_y_x)

    H_transpose_A = np.zeros(flattened_dim, dtype=float64)
    for i in range(y.shape[0]):
        point = H[i]
        H_transpose_A[point] = A[i]
        H_R_H[point] = R_inv[i]

    C[:] = 1 / (B_inv[:] + H_R_H[:])

    x = xb + np.multiply(C, H_transpose_A)

    vo_x = np.reshape(x, (latit_dim, longit_dim))
    sph.forward_transform(vo_x, weights, legendre_matrix, vo_coef)
    sph.inverse_transform(vo_coef, legendre_matrix, longitudes, vo_x)
    x = vo_x.flatten()

    vo_3DVAR[:] = vo_x[:]
