
"""
.. module:: module_basic_BVE.py
    :synopsis: It contains the functions used in the basic BVE espectral model.
.. moduleauthor:: Kereman Alonso (alonso.kerman5@gmail.com)
"""

import numpy as np
from numpy import float64, arcsin
from math import sqrt as sqrt, factorial as fac, pi
from cmath import exp as cexp
import numba
from scipy.special import roots_legendre, lpmn
from scipy.fftpack import fft, ifft

from matplotlib import pyplot as plt, cm


def my_associated_legendre(latitudes, legendre_matrix, beta_matrix):
    """
        This function computes the associated legendre matrix and beta matrix
        used to define spherical harmonics. It doesn't use scipy functions.

    """
    # Get the triangula truncation parameter M
    M = legendre_matrix.shape[0] - 1
    # Calculate the beta values
    beta(beta_matrix)
    latit_dim = latitudes.shape[0]
    for latit in range(latit_dim):
        # Get the value of the latitude parameter
        mu = latitudes[latit]
        # Store P0,0
        legendre_matrix[0, 0, latit] = 1

        if (abs(mu) > 1e-6):
            # Calculate and store all Pm,n from n=1 up to n=M order
            for n in range(1, M + 1):
                # Get the value of Pn-1,n-1 from matrix
                previous_P = legendre_matrix[n-1, n-1+(n-1), latit]
                # Calculate and store the value of Pn,n
                legendre_matrix[n, n+(n), latit] =\
                    - (2 * (n - 1) + 1) *\
                    sqrt(1 - mu ** 2) *\
                    beta_matrix[n, n] / beta_matrix[n-1, n-1] *\
                    previous_P * (-1)**(n-1) * (-1)**(n)

                # Calculte the values from Pn-1,n to P0,n
                for m in range(n-1, -1, -1):
                    P = sqrt(1 - mu**2) *\
                        legendre_matrix[n, m+1+(n), latit] * (-1)**(m+1)
                    P += (n + m) * beta_matrix[n, m+1] /\
                        beta_matrix[n-1, m] *\
                        legendre_matrix[n-1, m+(n-1), latit] * (-1)**(m)
                    P *= 1 / ((n-m) * mu) * beta_matrix[n, m] /\
                        beta_matrix[n, m+1]
                    legendre_matrix[n, m+(n), latit] = P * (-1)**(m)
                # Calculate the values from P-1,n to P-n,n
                for m in range(-1, -n-1, -1):
                    legendre_matrix[n, m+(n), latit] = (-1) ** (-m) *\
                        legendre_matrix[n, -m+(n), latit]
        else:
            for n in range(1, M + 1):
                for m in range(n, -1, -1):
                    if ((n-m) % 2 == 1):
                        legendre_matrix[n, m+(n), latit] = 0
                    else:
                        s = (n - m) / 2
                        legendre_matrix[n, m+(n), latit] =\
                            (-1)**s * fac(2*s + 2*m) /\
                            (2**n * fac(s) * fac(s+m)) *\
                            beta_matrix[n, m]
                for m in range(-1, -n-1, -1):
                    legendre_matrix[n, m+(n), latit] = (-1) ** (-m) *\
                        legendre_matrix[n, -m+(n), latit]


def associated_legendre(latitudes, legendre_matrix, beta_matrix):
    """
        This function computes the associated legendre matrix and beta matrix
        used to define spherical harmonics. It does use scipy functions.

    """
    # Get the triangula truncation parameter M
    M = legendre_matrix.shape[0] - 2
    # Calculate the beta values
    beta(beta_matrix)
    latit_dim = latitudes.shape[0]
    for latit in range(latit_dim):
        # Get the value of the latitude parameter
        mu = latitudes[latit]

        # Use scipy library to get the values of associated legendre
        # polynomials
        scipy_values = lpmn(M+2, M+2, mu)[0]

        # Transform the scipy values to get the values of the associated
        # legendre polynomials according to the chosen definition
        for n in range(M + 2):
            # Get the values from Pn,n to P0,n
            for m in range(n, -1, -1):
                legendre_matrix[n, m+(n), latit] = scipy_values[m, n] *\
                    (-1.0)**m * beta_matrix[n, m]
            # Get the values from P-1,n to P-n,n
            for m in range(-1, -n-1, -1):
                legendre_matrix[n, m+(n), latit] = (-1.0) ** (-m) *\
                    legendre_matrix[n, -m+(n), latit]


def beta(beta_matrix):
    """
        This function computes the beta matrix used to define spherical
        harmonics.

    """
    M = beta_matrix.shape[0] - 2
    for n in range(M+2):
        for m in range(n+1):
            A = 1.0
            for term in range(n-m+1, n+m+1):
                A *= term
            beta_matrix[n, m] = sqrt((2.0 * n + 1) / A)


@numba.jit(nopython=True)
def longit_partial(spectral_coef, result_coef):
    """
        This function computes the spectral coeficients that result from
        applying the longitudinal partial derivative.

    """
    M = spectral_coef.shape[0] - 1
    for n in range(M+1):
        for m in range(-n, n+1):
            result_coef[n, m+(n)] =\
                spectral_coef[n, m+(n)] *\
                1j * m


@numba.jit(nopython=True)
def laplacean(spectral_coef, a, result_coef):
    """
        This function computes the spectral coeficients that result from
        applying the Laplacean.

    """
    M = spectral_coef.shape[0] - 1
    for n in range(M+1):
        for m in range(-n, n+1):
            result_coef[n, m+(n)] =\
                spectral_coef[n, m+(n)] *\
                - n * (n+1) / a ** 2


@numba.jit(nopython=True)
def poisson(spectral_coef, a, result_coef):
    """
        This function computes the spectral coeficients that result from
        solving Poisson's equation.

    """
    M = spectral_coef.shape[0] - 1
    for n in range(1, M+1):
        for m in range(-n, n+1):
            result_coef[n, m+(n)] =\
                spectral_coef[n, m+(n)] *\
                - a ** 2 / (n * (n+1))


@numba.jit(nopython=True)
def quasi_latit_partial(spectral_coef, beta_matrix, result_coef):
    """
        This function computes the spectral coeficients that result from
        applying the modified latitudinal partial derivative.

    """
    M = spectral_coef.shape[0] - 1
    for n in range(M+2):
        for m in range(n+1):
            A = 0
            if (n < M):
                A += C2(m, n+1, beta_matrix) * spectral_coef[n+1, m+(n+1)]

            if (abs(m) <= n - 1):
                A += C1(m, n-1, beta_matrix) * spectral_coef[n-1, m+(n-1)]

            result_coef[n, m+(n)] = A
        if (n > 0):
            for m in range(-1, -n-1, -1):
                result_coef[n, m+(n)] = (-1) ** (-m) *\
                    result_coef[n, -m+(n)].conjugate()


@numba.jit(nopython=True)
def C1(m, n, beta_matrix):
    return n * (n-m+1) / (2*n+1) * beta_matrix[n, m] /\
                    beta_matrix[n+1, m]


@numba.jit(nopython=True)
def C2(m, n, beta_matrix):
    return - (n+m) * (n+1) / (2*n+1) * beta_matrix[n, m] /\
                    beta_matrix[n-1, m]


def coordinates_and_weights(longit_dim, latit_dim):
    """
        This function computes the longitude and latitude values of the
        physical grid. It also computes the weights used in Gaussian
        quadrature.

    """
    latitudes, weights = roots_legendre(latit_dim)
    longit_step = 2 * pi / longit_dim
    longitudes = np.array([longit_step * i - pi for i in range(longit_dim)],
                          dtype=np.float64)
    return longitudes, latitudes, weights


def inverse_transform(spectral_coef, legendre_matrix, longitudes, grid_values):
    """
        This function computes the physical grid values of a variable from
        its spectral coefficient values.

    """
    longit_dim = grid_values.shape[1]
    latit_dim = grid_values.shape[0]
    M = spectral_coef.shape[0] - 1
    b_m = np.zeros((latit_dim, 2*M+1), dtype=np.complex)

    # Get the values of b_m (a matrix that contains the arrays that
    # will be used by inverse fast Fourier transform).
    for latit in range(latit_dim):
        for m in range(-M, M+1):
            g_value = 0
            for n in range(abs(m), M+1):
                g_value += spectral_coef[n, m+(n)] *\
                    legendre_matrix[n, m+(n), latit]
            b_m[-latit+latit_dim-1, m+M] = g_value * cexp(-1j*m*pi)

    # Apply inverse fast fourier transform to get the grid point values
    for latit in range(latit_dim):
        y = np.roll(np.append(b_m[-latit+latit_dim-1, :],
                              np.zeros(M, dtype=np.complex)), -M)
        N = longit_dim
        grid_values[-latit+latit_dim-1, :] = np.real(ifft(y, N))


def forward_transform(grid_values, weights, legendre_matrix, spectral_coef):
    """
        This function computes the spectral coefficients of a variable from
        its physical grid values.

    """
    M = spectral_coef.shape[0] - 1
    latit_dim = grid_values.shape[0]
    longit_dim = grid_values.shape[1]
    a_m = np.zeros((latit_dim, M+1), dtype=np.complex)

    # Use the fast fourier transform to calculate the longitudinal integral
    for latit in range(latit_dim):
        x = grid_values[-latit+latit_dim-1, :]
        N = longit_dim
        a_m[-latit+latit_dim-1, :] = fft(x, N)[:M+1]

    # Use Gauss-Legendre quadrature to calculate the latitudinal integral
    for n in range(M+1):
        for m in range(n+1):
            gauss_quadrature = 0
            for latit in range(latit_dim):
                gauss_quadrature += weights[latit] *\
                    a_m[-latit+latit_dim-1, m] * cexp(1j*m*pi) *\
                    legendre_matrix[n, m+(n), latit]
            spectral_coef[n, m+(n)] = 1 / 2 * gauss_quadrature
        if (n > 0):
            for m in range(-1, -n-1, -1):
                spectral_coef[n, m+(n)] = (-1) ** (-m) *\
                    spectral_coef[n, -m+(n)].conjugate()


@numba.jit(nopython=True)
def get_G_grid(longit_psi_grid, latit_psi_grid, longit_lapl_psi_grid,
               latit_lapl_psi_grid, latitudes, G_grid):
    """
        This function computes the values of G in the physical grid.

    """
    latit_dim = longit_psi_grid.shape[0]
    longit_dim = longit_psi_grid.shape[1]
    for latit in range(latit_dim):
        for longit in range(longit_dim):
            A = 1 / (1 - latitudes[latit]**2)
            i = - latit + latit_dim - 1
            G_grid[i, longit] = A *\
                ((longit_psi_grid[i, longit] *
                  latit_lapl_psi_grid[i, longit])
                 - (longit_lapl_psi_grid[i, longit] *
                    latit_psi_grid[i, longit]))


@numba.jit(nopython=True)
def coef_time_forward(t_step, omega, a, G_coef, psi_coef_past,
                      psi_coef_current):
    """
        This function applies forward finite diferencing to get the
        streamfunction coefficients of at the next future time.

    """
    M = psi_coef_past.shape[0] - 1
    psi_coef_current[0, 0] = psi_coef_past[0, 0]

    # tau: e-folding time scale for the diffusion at the smallest wavelength
    tau = 7 * 3600
    # 2*hs: order of hiperdiffusion
    hs = 2
    # K: hyperdiffusion magnitude
    K = (a**2/(M*(M+1))) ** hs / tau * 9
    sign = (-1)**hs

    for n in range(1, M+1):
        for m in range(n+1):
            psi_coef_current[n, m+(n)] = psi_coef_past[n, m+(n)] +\
                t_step * 1.0 / (n*(n+1)) *\
                ((2.0 * omega * 1j * m - sign * K *
                  (-n*(n+1)/a**2)**hs * n*(n+1)) *
                 psi_coef_past[n, m+(n)] -
                 G_coef[n, m+(n)])
        for m in range(-1, -n-1, -1):
            psi_coef_current[n, m+(n)] = (-1) ** (-m) *\
                psi_coef_current[n, -m+(n)].conjugate()


@numba.jit(nopython=True)
def coef_time_centered(t_step, omega, a, G_coef, psi_coef_past,
                       psi_coef_current, psi_coef_future):
    """
        This function applies centered finite diferencing to get the
        streamfunction coefficients of at the next future time.

    """
    M = psi_coef_past.shape[0] - 1

    # tau: e-folding time scale for the diffusion at the smallest wavelength
    tau = 7 * 3600
    # 2*hs: order of hiperdiffusion
    hs = 2
    # K: hyperdiffusion magnitude
    K = (a**2/(M*(M+1))) ** hs / tau * 9
    sign = (-1)**hs

    # Apply time centered scheme to integrate forward in time
    psi_coef_future[0, 0] = psi_coef_current[0, 0]
    for n in range(1, M+1):
        for m in range(n+1):
            psi_coef_future[n, m+(n)] = psi_coef_past[n, m+(n)] +\
                2.0 * t_step * 1 / (n*(n+1)) *\
                ((2.0 * omega * 1j * m - sign * K *
                  (-n*(n+1)/a**2)**hs * n*(n+1)) *
                 psi_coef_current[n, m+(n)] -
                 G_coef[n, m+(n)])
        for m in range(-1, -n-1, -1):
            psi_coef_future[n, m+(n)] = (-1) ** (-m) *\
                psi_coef_future[n, -m+(n)].conjugate()

    # Apply time filter
    r = 0.2
    for n in range(1, M+1):
        for m in range(n+1):
            psi_coef_current[n, m+(n)] =\
                (1 - 2*r) * psi_coef_current[n, m+(n)] +\
                r * (psi_coef_future[n, m+(n)] + psi_coef_past[n, m+(n)])
        for m in range(-1, -n-1, -1):
            psi_coef_current[n, m+(n)] = (-1) ** (-m) *\
                psi_coef_current[n, -m+(n)].conjugate()

    # Update coefficient matrices
    psi_coef_past[:] = psi_coef_current[:]
    psi_coef_current[:] = psi_coef_future[:]


def psi_forecast(year_month, t0, t_step, forecast_step, max_time,
                 omega, a, vorticity,
                 norm, M, target='forecasts_no_3DVAR', initial_t=0):
    """
        This function solves the BVE equation to compute the forecasts
        of vorticity.

    """
    # Calculate the latitude and longitude values of the grid points.
    # Calculate the Gauss-Legendre quadrature weights that correspond to the
    # latitude points.
    longit_dim = 3 * M + 1
    latit_dim = (3 * M + 1) // 2 if (3 * M + 1) % 2 == 0 else\
        (3 * M + 1) // 2 + 1
    longitudes, latitudes, weights =\
        coordinates_and_weights(longit_dim, latit_dim)

    # Get a matrix with the necessary values of associated Legendre polinomials
    # to transform back and forth between physical grid and spectral
    # coefficients
    legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
    beta_matrix = np.zeros((M+2, M+2))
    associated_legendre(latitudes, legendre_matrix, beta_matrix)

    # Get the values of the streamfunction in the grid
    psi_grid = np.zeros((latit_dim, longit_dim), dtype=float64)
    vorticity_to_psi(vorticity, longitudes, latitudes, weights, a, M,
                     psi_grid)

    # Make forward transform to get the spectral coeficients of psi_grid
    # named psi_coef_past
    psi_coef_past = np.zeros((M+1, 2*M+1), dtype=np.complex)
    forward_transform(psi_grid, weights, legendre_matrix, psi_coef_past)

    # Calculate the spectral coeficients of the laplacean of psi
    lapl_psi_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
    laplacean(psi_coef_past, a, lapl_psi_coef)

    # Calculate the spectral coeficients of the longitudinal derivative of
    # psi. Calculate the coresponding grid values.
    longit_psi_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
    longit_partial(psi_coef_past, longit_psi_coef)
    longit_psi_grid = np.zeros((latit_dim, longit_dim), dtype=np.float64)
    inverse_transform(longit_psi_coef, legendre_matrix, longitudes,
                      longit_psi_grid)

    # Calculate the spectral coeficients of the latitudinal derivative of psi
    # times (mu**2-1). Calculate the coresponding grid values.
    latit_psi_coef = np.zeros((M+2, 2*(M+1)+1), dtype=np.complex)
    quasi_latit_partial(psi_coef_past, beta_matrix, latit_psi_coef)
    latit_psi_grid = np.zeros((latit_dim, longit_dim), dtype=np.float64)
    inverse_transform(latit_psi_coef, legendre_matrix, longitudes,
                      latit_psi_grid)

    # Calculate the spectral coeficients of the longitudinal derivative of
    # laplacean(psi). Calculate the coresponding grid values.
    longit_lapl_psi_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
    longit_partial(lapl_psi_coef, longit_lapl_psi_coef)
    longit_lapl_psi_grid = np.zeros((latit_dim, longit_dim), dtype=np.float64)
    inverse_transform(longit_lapl_psi_coef, legendre_matrix, longitudes,
                      longit_lapl_psi_grid)

    # Calculate the spectral coeficients of the latitudinal derivative of
    # laplacean(psi) times (mu**2-1). Calculate the coresponding grid values.
    latit_lapl_psi_coef = np.zeros((M+2, 2*(M+1)+1), dtype=np.complex)
    quasi_latit_partial(lapl_psi_coef, beta_matrix, latit_lapl_psi_coef)
    latit_lapl_psi_grid = np.zeros((latit_dim, longit_dim), dtype=np.float64)
    inverse_transform(latit_lapl_psi_coef, legendre_matrix, longitudes,
                      latit_lapl_psi_grid)

    # Get the grid values of the C term (this term is related to the
    # Jacobian term of the BVE).
    G_grid = np.zeros((latit_dim, longit_dim), dtype=np.float64)
    get_G_grid(longit_psi_grid, latit_psi_grid, longit_lapl_psi_grid,
               latit_lapl_psi_grid, latitudes, G_grid)

    # Get the spectral coeficient values of the C term.
    G_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
    forward_transform(G_grid, weights, legendre_matrix, G_coef)

    # Apply finite difference forward scheme to get the spectral coeficients
    # of psi after t_step time has pased.
    psi_coef_current = np.zeros((M+1, 2*M+1), dtype=np.complex)
    coef_time_forward(t_step, omega, a, G_coef, psi_coef_past,
                      psi_coef_current)

    # Create psi_coef_future placeholder matrix to be able to use finite
    # difference centered scheme.
    psi_coef_future = np.zeros((M+1, 2*M+1), dtype=np.complex)

    # Loop to get the values of psi at different times.
    for t in range(2, max_time // t_step + 1):
        # Get the spectral coefficients of the C term.
        laplacean(psi_coef_current, a, lapl_psi_coef)

        longit_partial(psi_coef_current, longit_psi_coef)
        inverse_transform(longit_psi_coef, legendre_matrix, longitudes,
                          longit_psi_grid)

        quasi_latit_partial(psi_coef_current, beta_matrix, latit_psi_coef)
        inverse_transform(latit_psi_coef, legendre_matrix, longitudes,
                          latit_psi_grid)

        longit_partial(lapl_psi_coef, longit_lapl_psi_coef)
        inverse_transform(longit_lapl_psi_coef, legendre_matrix, longitudes,
                          longit_lapl_psi_grid)

        quasi_latit_partial(lapl_psi_coef, beta_matrix, latit_lapl_psi_coef)
        inverse_transform(latit_lapl_psi_coef, legendre_matrix, longitudes,
                          latit_lapl_psi_grid)

        get_G_grid(longit_psi_grid, latit_psi_grid, longit_lapl_psi_grid,
                   latit_lapl_psi_grid, latitudes, G_grid)
        forward_transform(G_grid, weights, legendre_matrix, G_coef)

        # Apply finite difference centered scheme to get the spectral
        # coeficients of psi after t_step time has pased.

        coef_time_centered(t_step, omega, a, G_coef, psi_coef_past,
                           psi_coef_current, psi_coef_future)

        # If the time corresponds to a forecast time (a time for which we are
        # interested in knowing the values of psi_grid), the values of
        # psi on the grid are calculated and stored in a csv file
        if (t * t_step % forecast_step == 0):
            print(t*t_step / 3600 + initial_t)
            inverse_transform(psi_coef_current, legendre_matrix, longitudes,
                              psi_grid)

            # Plot vorticity 2D color, and store in csv file
            p_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
            psi_to_vorticity(psi_grid, longitudes,
                             latitudes, weights, a, M, p_vorticity)
            
            latitudes_plot = - arcsin(latitudes)
            fig = plt.figure(t, dpi=150)
            color_plot = plt.pcolormesh(longitudes, latitudes_plot,
                                        p_vorticity*1e5,
                                        cmap=cm.hot,
                                        norm=norm)
            fig.colorbar(color_plot)

            if target == 'forecasts_no_3DVAR':
                write_vo_csv(p_vorticity, year_month, t0 // 4 + 1,
                             int(t*t_step / 3600))

                fig.suptitle(r'$t = {}h$'.format(int(t*t_step / 3600)))
                plt.show()

            elif target == 'B_computation':
                write_vo_csv_B_3DVAR(p_vorticity, year_month, t0 // 4 + 1,
                                   6 * (t0 % 4),
                                   int(t*t_step / 3600))
                fig.suptitle('vo_{}_{}_{}_{}'.format(
                    year_month, t0 // 4 + 1, 6 * (t0 % 4),
                    int(t*t_step / 3600)))
                plt.show()
            
            elif target == 'forecasts_with_3DVAR':
                write_vo_csv_forecast_3DVAR(p_vorticity, year_month, 
                                            t0 // 4 + 1,
                                            6 * (t0 % 4),
                                            int(t*t_step / 3600) + initial_t)
                fig.suptitle('vo_{}_{}_{}_{}'.format(
                    year_month, t0 // 4 + 1, 6 * (t0 % 4),
                    int(t*t_step / 3600) + initial_t))
                plt.show()


def vorticity_to_psi(vorticity, longitudes, latitudes, weights, a, M, psi):
    """
        This function computes stramfunction values from vorticity values.

    """

    latit_dim = latitudes.shape[0]
    # Get a matrix with the necessary values of associated Legendre polinomials
    # to transform back and forth between physical grid and spectral
    # coefficients
    legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
    beta_matrix = np.zeros((M+2, M+2))
    associated_legendre(latitudes, legendre_matrix, beta_matrix)

    # Make forward transform to get the spectral coeficients of vorticity
    vo_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
    forward_transform(vorticity, weights, legendre_matrix, vo_coef)

    # Calculate the spectral coeficients of the result of poissons equation
    psi_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
    poisson(vo_coef, a, psi_coef)

    # Make inverse transform to to get the grid values of psi
    inverse_transform(psi_coef, legendre_matrix, longitudes, psi)


def psi_to_vorticity(psi, longitudes, latitudes, weights, a, M, vorticity):
    """
        This function computes vorticity values from streamfunction values.

    """

    latit_dim = latitudes.shape[0]
    # Get a matrix with the necessary values of associated Legendre polinomials
    # to transform back and forth between physical grid and spectral
    # coefficients
    legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
    beta_matrix = np.zeros((M+2, M+2))
    associated_legendre(latitudes, legendre_matrix, beta_matrix)

    # Make forward transform to get the spectral coeficients of psi
    psi_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
    forward_transform(psi, weights, legendre_matrix, psi_coef)

    # Calculate the spectral coeficients of the result of laplacean
    vo_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
    laplacean(psi_coef, a, vo_coef)

    # Make inverse transform to to get the grid values of psi
    inverse_transform(vo_coef, legendre_matrix, longitudes, vorticity)


def psi_to_u(psi, longitudes, latitudes, weights, a, M, u):
    """
        This function computes u wind component values from streamfunction
        values.

    """

    latit_dim = latitudes.shape[0]
    longit_dim = longitudes.shape[0]
    # Get a matrix with the necessary values of associated Legendre polinomials
    # to transform back and forth between physical grid and spectral
    # coefficients
    legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
    beta_matrix = np.zeros((M+2, M+2))
    associated_legendre(latitudes, legendre_matrix, beta_matrix)

    # Make forward transform to get the spectral coeficients of psi
    psi_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
    forward_transform(psi, weights, legendre_matrix, psi_coef)

    # Calculate the spectral coeficients of v
    u_coef = np.zeros((M+2, 2*(M+1)+1), dtype=np.complex)
    quasi_latit_partial(psi_coef, beta_matrix, u_coef)
    inverse_transform(u_coef, legendre_matrix, longitudes, u)

    for latit in range(latit_dim):
        for longit in range(longit_dim):
            i = - latit + latit_dim - 1
            u[i, longit] = 1 / (a * sqrt(1 - latitudes[latit]**2)) *\
                u[i, longit]


def psi_to_v(psi, longitudes, latitudes, weights, a, M, v):
    """
        This function computes v wind component values from streamfunction
        values.

    """

    latit_dim = latitudes.shape[0]
    longit_dim = longitudes.shape[0]
    # Get a matrix with the necessary values of associated Legendre polinomials
    # to transform back and forth between physical grid and spectral
    # coefficients
    legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
    beta_matrix = np.zeros((M+2, M+2))
    associated_legendre(latitudes, legendre_matrix, beta_matrix)

    # Make forward transform to get the spectral coeficients of psi
    psi_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
    forward_transform(psi, weights, legendre_matrix, psi_coef)

    # Calculate the spectral coeficients of v
    v_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
    longit_partial(psi_coef, v_coef)
    inverse_transform(v_coef, legendre_matrix, longitudes, v)

    for latit in range(latit_dim):
        for longit in range(longit_dim):
            i = - latit + latit_dim - 1
            v[i, longit] = 1 / (a * sqrt(1 - latitudes[latit]**2)) *\
                v[i, longit]


def get_planetary_vo(omega, latitudes, f):
    for latit in range(f.shape[0]):
        f[latit] = 2 * omega * latitudes[latit]


def sub_planetary_vo(f, vorticity):
    latit_dim = vorticity.shape[0]
    longit_dim = vorticity.shape[1]
    for latit in range(latit_dim):
        for longit in range(longit_dim):
            vorticity[-latit+latit_dim-1, longit] -= f[latit]


def add_planetary_vo(f, vorticity):
    latit_dim = vorticity.shape[0]
    longit_dim = vorticity.shape[1]
    for latit in range(latit_dim):
        for longit in range(longit_dim):
            vorticity[-latit+latit_dim-1, longit] += f[latit]


def sphere_plot(longitudes, latitudes, psi, t):
    # Write '%matplotlib qt' in the command line to make the graphs in
    # separate windows (this allows to move the graph).
    # Write '%matplotlib inline' to return to inline graph mode

    theta = - np.arcsin(latitudes) + pi / 2
    phi = longitudes + pi
    theta, phi = np.meshgrid(-theta+pi, phi, indexing='ij')

    # The Cartesian coordinates of the unit sphere
    x = np.sin(theta) * np.cos(phi)
    y = np.sin(theta) * np.sin(phi)
    z = np.cos(theta)

    # Normalize psi_grid to [0,1]
    psi_max, psi_min = psi.max(), psi.min()
    n_psi = (psi - psi_min)/(psi_max - psi_min)

    # Set the aspect ratio to 1 so our sphere looks spherical
    fig = plt.figure(t, figsize=plt.figaspect(1.))
    ax = fig.add_subplot(111, projection='3d')
    ax.plot_surface(-x, -y, z,  rstride=1, cstride=1,
                    facecolors=cm.prism(n_psi))
    # ax.contour(x, y, z, n_psi, extend3D=True, stride=1, levels=10,
    #            colors='black')
    # Turn off the axis planes
    ax.set_axis_off()
    plt.show()


def write_vo_csv(vorticity, year_month, day, t):
    with open('spherical_csv\\vorticity_{}_{}_{}.csv'.format(
            year_month, day, t),
              "w", newline='') as csvfile:
        np.savetxt(csvfile, vorticity, delimiter=",")


def write_vo_csv_B_3DVAR(vorticity, year_month, day, hour, t):
    with open('spherical_csv\\B_3DVAR\\vo_{}_{}_{}_{}.csv'.format(
            year_month, day, hour, t),
              "w", newline='') as csvfile:
        np.savetxt(csvfile, vorticity, delimiter=",")


def write_vo_csv_forecast_3DVAR(vorticity, year_month, day, hour, t):
    with open('spherical_csv\\forecast_3DVAR\\vo_{}_{}_{}_{}.csv'.format(
            year_month, day, hour, t),
              "w", newline='') as csvfile:
        np.savetxt(csvfile, vorticity, delimiter=",")


def read_vo_csv(year_month, day, t, vorticity):
    with open('spherical_csv\\vorticity_{}_{}_{}.csv'.format(
            year_month, day, t),
              "r", newline='') as csvfile:
        vorticity[:] = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")[:]
