
"""
.. module:: validation_3DVAR.py
    :synopsis: It contains the scripts used in the validation process of the 
    BVE model with 3DVAR.
.. moduleauthor:: Kereman Alonso (alonso.kerman5@gmail.com)
"""

import numpy as np
from numpy import float64, arcsin, sqrt
from sklearn.linear_model import LinearRegression

from matplotlib import pyplot as plt, cm, colors
from matplotlib.lines import Line2D
from mpl_toolkits.mplot3d import Axes3D

from netCDF4 import Dataset

import get_external_measurements as ext_measurement
import module_basic_BVE as sph
import module_3DVAR

# omega: angular velocity of Earth in rad/s.
omega = 7.2921e-5
# a: radius of Earth in metres.
a = 6.3781e6

# Get the analysis data
year = '2019'
month = '07'
an = Dataset('..\\..\\data\\an_{}_{}.nc'.format(year, month), 'r')

# t0: index of initial time in ERA Interim analysis time array.
# t0 = 0 corresponds to the first day of a month at 00:00
day = 20
t0 = (day - 1) * 4

# Select pressure
# # pressure: index of pressure in ERA5 pressure array. pressure = 2
# # corresponds to 500 hPa
pressure = 2

# Set initial vorticity parameters
# # order: order of antialiasing
order = 2
# # M: spherical harmonic truncation parameter
M = 63

longit_dim = 3 * M + 1
latit_dim = (3 * M + 1) // 2 if (3 * M + 1) % 2 == 0 else\
    (3 * M + 1) // 2 + 1
flattened_dim = latit_dim * longit_dim
longitudes, latitudes, weights =\
    sph.coordinates_and_weights(longit_dim, latit_dim)
latitudes_plot = -arcsin(latitudes)

vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
t_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
vo_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)

# Set forecast parameters
legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
beta_matrix = np.zeros((M+2, M+2), dtype=float64)

sph.associated_legendre(latitudes, legendre_matrix, beta_matrix)


"""
Compute and save BVE forecast to be used in the computation of B.
"""
# # Get the analysis data
# year = '2019'
# month = '07'
# an = Dataset('..\\..\\data\\an_{}_{}.nc'.format(year, month), 'r')

# # ============================= #
# # --------24h forecasts-------- #

# # Set forecast times
# t_step = 900
# forecast_step = 24 * 3600
# max_time = 24 * 3600

# for day in range(1, 31):
#     # t0: index of initial time in ERA Interim analysis time array.
#     # t0 = 0 corresponds to the first day of a month at 00:00
#     t0 = (day - 1) * 4

#     # Get initial vorticity matrix from the analysis data
#     ext_measurement.get_vorticity_matrix(an, t0, pressure, latitudes,
#                                           longitudes, order, vorticity)
#     norm = cm.colors.Normalize(vmin=vorticity.min(),
#                                 vmax=vorticity.max())
#     # Make forward and inverse transform of vorticity to get the vorticity
#     # that can be represented with M modes
#     sph.forward_transform(vorticity, weights, legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix, longitudes, t_vorticity)

#     # Compute and save forecast
#     sph.psi_forecast('{}_{}'.format(year, month), t0, t_step, forecast_step,
#                       max_time, omega, a,
#                       t_vorticity,
#                       norm, M, target='B_computation')

# # ============================= #

# # ============================= #
# # --------12h forecasts-------- #

# # Set forecast times
# t_step = 900
# forecast_step = 12 * 3600
# max_time = 12 * 3600

# for day in range(1, 31):
#     # t0: index of initial time in ERA Interim analysis time array.
#     # t0 = 0 corresponds to the first day of a month at 00:00
#     t0 = (day - 1) * 4 + 2

#     # Get initial vorticity matrix from the analysis data
#     ext_measurement.get_vorticity_matrix(an, t0, pressure, latitudes,
#                                           longitudes, order, vorticity)

#     # Get initial vorticity matrix from the analysis data
#     ext_measurement.get_vorticity_matrix(an, t0, pressure, latitudes,
#                                           longitudes, order, vorticity)
#     norm = cm.colors.Normalize(vmin=vorticity.min(),
#                                 vmax=vorticity.max())
#     # Make forward and inverse transform of vorticity to get the vorticity
#     # that can be represented with M modes
#     sph.forward_transform(vorticity, weights, legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix, longitudes, t_vorticity)

#     # Compute and save forecast
#     sph.psi_forecast('{}_{}'.format(year, month), t0, t_step, forecast_step,
#                       max_time, omega, a,
#                       t_vorticity,
#                       norm, M, target='B_computation')

# # ============================= #

# # ============================= #
# # --------6h forecasts-------- #

# # Set forecast times
# t_step = 900
# forecast_step = 6 * 3600
# max_time = 6 * 3600

# for day in range(1, 31):
#     # t0: index of initial time in ERA Interim analysis time array.
#     # t0 = 0 corresponds to the first day of a month at 00:00
#     t0 = (day - 1) * 4 + 3

#     # Get initial vorticity matrix from the analysis data
#     ext_measurement.get_vorticity_matrix(an, t0, pressure, latitudes,
#                                           longitudes, order, vorticity)

#     # Get initial vorticity matrix from the analysis data
#     ext_measurement.get_vorticity_matrix(an, t0, pressure, latitudes,
#                                           longitudes, order, vorticity)
#     norm = cm.colors.Normalize(vmin=vorticity.min(),
#                                 vmax=vorticity.max())
#     # Make forward and inverse transform of vorticity to get the vorticity
#     # that can be represented with M modes
#     sph.forward_transform(vorticity, weights, legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix, longitudes, t_vorticity)

#     # Compute and save forecast
#     sph.psi_forecast('{}_{}'.format(year, month), t0, t_step, forecast_step,
#                       max_time, omega, a,
#                       t_vorticity,
#                       norm, M, target='B_computation')

# # ============================= #

"""
Compute and save B.
"""
# year_months = ['2019_07']
# module_3DVAR.compute_B_24(year_months, latit_dim, longit_dim)
# module_3DVAR.compute_B_12(year_months, latit_dim, longit_dim)

"""
Read and print B.
"""
# B = np.zeros(flattened_dim, dtype=float64)
# module_3DVAR.read_B(24, B)
# print(B[100])
# print(B[10000])
# print(max(B))
# module_3DVAR.read_B(12, B)
# print(B[100])
# print(B[10000])
# print(max(B))

"""
Plot and save BVE forecast with 3DVAR data assimilation.
"""

# # Set forecast times
# t_step = 900
# forecast_step = 6 * 3600
# max_time = 48 * 3600

# # Get initial vorticity matrix from the analysis data
# ext_measurement.get_vorticity_matrix(an, t0, pressure, latitudes,
#                                       longitudes, order, vorticity)
# norm = cm.colors.Normalize(vmin=vorticity.min()*1e5,
#                             vmax=vorticity.max()*1e5)

# # Make forward and inverse transform of vorticity to get the vorticity
# # that can be represented with M modes
# sph.forward_transform(vorticity, weights, legendre_matrix, vo_coef)
# sph.inverse_transform(vo_coef, legendre_matrix, longitudes, t_vorticity)

# # # Plot truncated initial vorticity, and save
# sph.write_vo_csv_forecast_3DVAR(t_vorticity, '{}_{}'.format(year, month),
#                                 day, 0, 0)

# fig = plt.figure(3, dpi=150)
# color_plot = plt.pcolormesh(longitudes, latitudes_plot, t_vorticity*1e5,
#                             cmap=cm.hot,
#                             norm=norm)
# fig.colorbar(color_plot)
# fig.suptitle(r'$t = {}h$'.format(0))
# plt.show()

# # Set "observation" points and get H matrix
# ob_point_num = int(flattened_dim * 0.2)
# H = np.zeros(ob_point_num, dtype=int)
# module_3DVAR.get_H(ob_point_num, latit_dim, longit_dim, H)

# # Set 3DVAR step: apply 3DVAR assimilation every 12h or every 24h
# step_3DVAR = 12

# # Integrte forward in time applying 3DVAR data assimilation every step_3DVAR h.
# t0_3DVAR = t0
# max_time_3DVAR = step_3DVAR * 3600
# vo_an = np.zeros((latit_dim, longit_dim), dtype=float64)
# vo_3DVAR = np.zeros((latit_dim, longit_dim), dtype=float64)
# vo_forecast_0 = np.zeros((latit_dim, longit_dim), dtype=float64)

# vo_forecast_0[:] = t_vorticity[:]
# initial_t = 0

# for _ in range(max_time // max_time_3DVAR):

#     # Make step_3DVAR h forecast storing values every 6h
#     sph.psi_forecast('{}_{}'.format(year, month), t0,
#                       t_step, forecast_step,
#                       max_time_3DVAR, omega, a,
#                       vo_forecast_0,
#                       norm, M, target='forecasts_with_3DVAR',
#                       initial_t=initial_t)

#     # Get the vorticity of the step_3DVAR h forecast
#     with open('spherical_csv\\forecast_3DVAR\\vo_{}_{}_{}_{}.csv'.format(
#             '{}_{}'.format(year, month), day, 6 * (t0 % 4),
#             initial_t+step_3DVAR),
#             "r", newline='') as csvfile:
#         vo_forecast_step_3DVAR = np.loadtxt(csvfile,
#                                             dtype=np.float64, delimiter=",")

#     # Get the vorticity of the step_3DVAR h "observation"
#     ext_measurement.get_vorticity_matrix(an, 
#                                           t0_3DVAR + step_3DVAR // 6,
#                                           pressure,
#                                           latitudes,
#                                           longitudes, order, vo_an)
#     sph.forward_transform(vo_an, weights, legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix, longitudes, vo_an)

#     # Compute and store the vorticity with 3DVAR data assimilation
#     module_3DVAR.apply_3DVAR(vo_forecast_step_3DVAR, vo_an,
#                               latit_dim, longit_dim, weights,
#                               vo_coef, legendre_matrix, longitudes,
#                               ob_point_num, H, step_3DVAR,
#                               vo_3DVAR)
    
#     # print(np.average(np.absolute(vo_3DVAR-vo_forecast_step_3DVAR)))
#     # print(np.average(np.absolute(vo_3DVAR-vo_an)))
#     print((vo_3DVAR-vo_forecast_step_3DVAR).max())
#     print((vo_3DVAR-vo_an).max())

#     sph.write_vo_csv_forecast_3DVAR(vo_3DVAR, '{}_{}'.format(year, month),
#                                     day, 6 * (t0 % 4), initial_t+step_3DVAR)

#     # Update initial time and vorticity values for next step_3DVAR h forecast
#     initial_t += step_3DVAR
#     t0_3DVAR += step_3DVAR // 6

#     vo_forecast_0[:] = vo_3DVAR[:]

# # Compute the rest of the forecast if max_time is not a multiple of
# # step_3DVAR h
# if ((max_time - max_time // max_time_3DVAR *
#       max_time_3DVAR) // forecast_step > 0):

#     sph.psi_forecast('{}_{}'.format(year, month), t0,
#                       t_step, forecast_step,
#                       (max_time - max_time // max_time_3DVAR 
#                         * max_time_3DVAR),
#                       omega, a,
#                       vo_forecast_0,
#                       norm, M, target='forecasts_with_3DVAR',
#                       initial_t=initial_t)

"""
Scatter plots: BVE with 3DVAR vs ERA Iterim analysis
"""

# t_range = 9  # how many sample times

# BVE_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
# ERA_an_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)

# t_array = []
# RR_BVE = []
# a_BVE = []
# b_BVE = []

# R_BVE_err = []
# RR_BVE_err = []
# a_BVE_err = []
# b_BVE_err = []

# RR_ERA_fc = []
# a_ERA_fc = []
# b_ERA_fc = []

# for t in range(t_range):
#     # Get ERA Interim analysis vorticity
#     ext_measurement.get_vorticity_matrix(an, t+t0, pressure, latitudes,
#                                           longitudes, order,
#                                           ERA_an_vorticity)

#     # # Make forward and inverse transform of vorticity to get the
#     # # vorticity that can be represented with M modes
#     sph.forward_transform(ERA_an_vorticity, weights,
#                           legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix,
#                           longitudes, ERA_an_vorticity)

#     # Get BVE 3DVAR vorticity matrix
#     with open('spherical_csv\\forecast_3DVAR\\vo_{}_{}_{}_{}.csv'.format(
#             '{}_{}'.format(year, month), day, 6 * (t0 % 4), t*6),
#             "r", newline='') as csvfile:
#         BVE_vorticity = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")

#     # Plot scatter plot and get Pearson correlation coefficient R
#     # # Flatten vorticity matrices
#     x = ERA_an_vorticity.flatten()
#     y_BVE = BVE_vorticity.flatten()

#     # # Get the values of R
#     R_BVE = np.corrcoef(x, y_BVE)[0, 1]

#     t_array.append(t*6)
#     RR_BVE.append(R_BVE**2)

#     fig = plt.figure(50+t, dpi=150)
#     plt.scatter(x, y_BVE, s=0.2)
#     left = -0.00015
#     right = 0.00015
#     plt.xlim(left, right)
#     plt.ylim(left, right)
#     plt.title(r'$t = {}h$'.format(t*6))
#     plt.xlabel('Analisi bortizitatea {}'.format(r'$(s^{-1})$'))
#     plt.ylabel('BVE bortizitatea {}'.format(r'$(s^{-1})$'))

#     # # Store y = a + b * x linear regression parameters
#     x_ax = x.reshape((-1, 1))

#     regression_BVE = LinearRegression()
#     regression_BVE.fit(x_ax, y_BVE)
#     a_BVE.append(regression_BVE.intercept_)
#     b_BVE.append(regression_BVE.coef_)

#     # ================================================ #
#     # --Compute confidence intervals using bootstrap-- #

#     indices = np.array([i for i in range(x.shape[0])], dtype=int)
#     bootstrap_iterations = 1000
#     bootstrap_matrix = np.zeros((bootstrap_iterations, 4), dtype=float64)
#     for i in range(bootstrap_iterations):
#         samples = np.random.choice(indices, size=indices.shape[0],
#                                     replace=True)
#         samples_x = np.array([x[indice] for indice in samples], dtype=float64)
#         samples_y = np.array([y_BVE[indice] for indice in samples],
#                               dtype=float64)

#         bootstrap_matrix[i, 0] = np.corrcoef(samples_x, samples_y)[0, 1]
#         bootstrap_matrix[i, 1] = bootstrap_matrix[i, 0] ** 2

#         regression_samples = LinearRegression()
#         samples_x_ax = samples_x.reshape((-1, 1))
#         regression_samples.fit(samples_x_ax, samples_y)
#         bootstrap_matrix[i, 2] = regression_samples.intercept_
#         bootstrap_matrix[i, 3] = regression_samples.coef_

#     R_BVE_err.append((np.percentile(bootstrap_matrix[:, 0], 2.5),
#                       np.percentile(bootstrap_matrix[:, 0], 50),
#                       np.percentile(bootstrap_matrix[:, 0], 97.5)))

#     RR_BVE_err.append((np.percentile(bootstrap_matrix[:, 1], 2.5),
#                         np.percentile(bootstrap_matrix[:, 1], 50),
#                         np.percentile(bootstrap_matrix[:, 1], 97.5)))
    
#     a_BVE_err.append((np.percentile(bootstrap_matrix[:, 2], 2.5),
#                       np.percentile(bootstrap_matrix[:, 2], 50),
#                       np.percentile(bootstrap_matrix[:, 2], 97.5)))
    
#     b_BVE_err.append((np.percentile(bootstrap_matrix[:, 3], 2.5),
#                       np.percentile(bootstrap_matrix[:, 3], 50),
#                       np.percentile(bootstrap_matrix[:, 3], 97.5)))
    
#     # =============================================== #

#     plt.plot(x_ax[:, 0], a_BVE[-1] + b_BVE[-1] * x_ax[:, 0], 'k',
#               linewidth=0.5)
#     plt.show()

# # Save R**2 BVE vs time
# with open('spherical_csv\\forecast_3DVAR\\R_3DVAR_{}_{}_{}.csv'
#           .format(year, month, day),
#           "w", newline='') as csvfile:
#     t_array = np.array(t_array, dtype=float64)
#     RR_BVE = np.array(RR_BVE, dtype=float64)
#     a_BVE = np.array(a_BVE, dtype=float64)
#     b_BVE = np.array(b_BVE, dtype=float64)
#     to_csv = np.column_stack((t_array, RR_BVE, a_BVE, b_BVE))
#     np.savetxt(csvfile, to_csv, delimiter=",")

# # # Save R, R**2, a, b with errors (BVE)
# # with open('spherical_csv\\forecast_3DVAR\\R_3DVAR_err_{}_{}_{}.csv'
# #           .format(year, month, day),
# #           "w", newline='') as csvfile:
# #     t_array = np.array(t_array, dtype=float64)
# #     R_BVE_err = np.array(R_BVE_err, dtype=(float64, 3))
# #     RR_BVE_err = np.array(RR_BVE_err, dtype=(float64, 3))
# #     a_BVE_err = np.array(a_BVE_err, dtype=(float64, 3))
# #     b_BVE_err = np.array(b_BVE_err, dtype=(float64, 3))
# #     to_csv = np.column_stack((t_array, R_BVE_err, RR_BVE_err,
# #                               a_BVE_err, b_BVE_err))
# #     np.savetxt(csvfile, to_csv, delimiter=",")


"""
R**2 vs time plots: BVE with and without 3DVAR
"""

# fig = plt.figure(101, dpi=250)
# for year_month in ['2018_12', '2019_01', '2019_07', '2019_08']:
#     for day_i in [1, 10, 20]:
#         with open('spherical_csv\\R_BVE_{}_{}.csv'.format(year_month, day_i),
#                   "r", newline='') as csvfile:
#             from_csv = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")
#             t_array = from_csv[:, 0]
#             RR_array = from_csv[:, 1]

#             p_BVE = plt.plot(
#                 t_array, RR_array, color='grey', lw=1, linestyle='--',
#                 label='BVE')

# for year_month in ['2018_12', '2019_01', '2019_07', '2019_08']:
#     for day_i in [1, 10, 20]:
#         with open('spherical_csv\\forecast_3DVAR\\R_3DVAR_{}_{}.csv'
#                   .format(year_month, day_i),
#                   "r", newline='') as csvfile:
#             from_csv = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")
#             t_array = from_csv[:, 0]
#             RR_array = from_csv[:, 1]

#             p_3DVAR = plt.plot(t_array, RR_array, color='r', lw=1,
#                                 label='BVE + 3DVAR')
            
# for year_month in ['2018_12', '2019_01', '2019_07', '2019_08']:
#     for day_i in [1, 10, 20]:
#         with open('spherical_csv\\R_ERA_fc_{}_{}.csv'.format(year_month,
#                                                               day_i),
#                   "r", newline='') as csvfile:
#             from_csv = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")
#             t_array = from_csv[:, 0]
#             RR_array = from_csv[:, 1]

#             p_ECMWF = plt.plot(t_array, RR_array, color='b', lw=1,
#                                 label='ECMWF')

# left = 0
# right = max(t_array)
# plt.xlim(left, right)
# plt.ylim(bottom=0, top=1.05)
# plt.xlabel('Denbora (h)')
# plt.ylabel(r'$R^2$')
# plt.legend([Line2D([0], [0], color='grey', lw=1, linestyle='--'),
#             Line2D([0], [0], color='r', lw=1),
#             Line2D([0], [0], color='b', lw=1)],
#            ['BVE', 'BVE + 3DVAR', 'ECMWF'], loc='lower left')
# plt.show()

"""
Compute climatology matrix
"""
# ERA_an_vo = np.zeros((latit_dim, longit_dim), dtype=float64)
# climatology_vo = np.zeros((latit_dim, longit_dim), dtype=float64)
# for t in range(84):
#     # Get ERA Interim analysis and forecast vorticity matrices
#     ext_measurement.get_vorticity_matrix(an, t+t0, pressure, latitudes,
#                                           longitudes, order,
#                                           ERA_an_vo)
#     climatology_vo += ERA_an_vo
# climatology_vo /= 84

# with open('spherical_csv\\ACC\\climatology.csv',
#           "w", newline='') as csvfile:
#     np.savetxt(csvfile, climatology_vo, delimiter=",")

"""
Compute and save ACC
"""
# t_range = 15  # how many sample times since t0

# BVE_vo = np.zeros((latit_dim, longit_dim), dtype=float64)
# BVE_3DVAR_vo = np.zeros((latit_dim, longit_dim), dtype=float64)
# ERA_fc_vo = np.zeros((latit_dim, longit_dim), dtype=float64)
# ERA_an_vo = np.zeros((latit_dim, longit_dim), dtype=float64)
# climatology_vo = np.zeros((latit_dim, longit_dim), dtype=float64)
# vo_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)

# # Get climatology vorticity matrix
# with open('spherical_csv\\ACC\\climatology.csv',
#           "r", newline='') as csvfile:
#     climatology_vo = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")

# c = climatology_vo.flatten()

# legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
# beta_matrix = np.zeros((M+2, M+2), dtype=float64)
# sph.associated_legendre(latitudes, legendre_matrix, beta_matrix)

# fc = Dataset('..\\..\\data\\ACC_fc_{}_{}.nc'.format(year, month), 'r')
# # t_fc is the time index of the 6h forecast vorticity in the fc.nc file.
# # Select t_fc = 0 for day 1 of the month, t_fc = 14 for day 10 and t_fc = 28
# # for day 20.
# t_fc = 28

# t_array = []
# ACC_BVE = []
# ACC_BVE_3DVAR = []
# ACC_ERA_fc = []

# for t in range(t_range):
    
#     t_array.append(t*12)
#     # Get ERA Interim analysis and forecast vorticity matrices
#     ext_measurement.get_vorticity_matrix(an, t*2+t0, pressure, latitudes,
#                                           longitudes, order,
#                                           ERA_an_vo)
#     if (t == 0):
#         ext_measurement.get_vorticity_matrix(an, t0, pressure, latitudes,
#                                               longitudes, order,
#                                               ERA_fc_vo)
#     else:
#         ext_measurement.get_vorticity_matrix(fc, t-1+t_fc, pressure,
#                                               latitudes,
#                                               longitudes, order,
#                                               ERA_fc_vo)

#     # # Make forward and inverse transform of vorticity to get the
#     # # vorticity that can be represented with M modes
#     sph.forward_transform(ERA_an_vo, weights,
#                           legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix,
#                           longitudes, ERA_an_vo)

#     sph.forward_transform(ERA_fc_vo, weights,
#                           legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix,
#                           longitudes, ERA_fc_vo)

#     # Get BVE vorticity matrix
#     sph.read_vo_csv('{}_{}'.format(year, month), t0 // 4 + 1,
#                     t*12, BVE_vo)

#     # Get BVE 3DVAR vorticity matrix
#     with open('spherical_csv\\forecast_3DVAR\\vo_{}_{}_{}_{}.csv'.format(
#             '{}_{}'.format(year, month), day, 6 * (t0 % 4), t*12),
#             "r", newline='') as csvfile:
#         BVE_3DVAR_vo = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")

#     o = ERA_an_vo.flatten() - c
#     f_BVE = BVE_vo.flatten() - c
#     f_BVE_3DVAR = BVE_3DVAR_vo.flatten() - c
#     f_ERA_fc = ERA_fc_vo.flatten() - c

#     # Get ACC_BVE
#     ACC = np.sum(np.multiply(f_BVE, o)) /\
#         sqrt(np.sum(np.square(o)) * np.sum(np.square(f_BVE)))
#     ACC_BVE.append(ACC)

#     # Get ACC_BVE_3DVAR
#     ACC = np.sum(np.multiply(f_BVE_3DVAR, o)) /\
#         sqrt(np.sum(np.square(o)) * np.sum(np.square(f_BVE_3DVAR)))
#     ACC_BVE_3DVAR.append(ACC)

#     # Get ACC_ERA_fc
#     ACC = np.sum(np.multiply(f_ERA_fc, o)) /\
#         sqrt(np.sum(np.square(o)) * np.sum(np.square(f_ERA_fc)))
#     ACC_ERA_fc.append(ACC)


# # Save ACC_BVE vs time
# with open('spherical_csv\\ACC\\ACC_BVE_{}_{}_{}.csv'.format(year, month, day),
#           "w", newline='') as csvfile:
#     t_array = np.array(t_array, dtype=float64)
#     ACC_BVE = np.array(ACC_BVE, dtype=float64)
#     to_csv = np.column_stack((t_array, ACC_BVE))
#     np.savetxt(csvfile, to_csv, delimiter=",")


# # Save ACC_BVE_3DVAR vs time
# with open('spherical_csv\\ACC\\ACC_BVE_3DVAR_{}_{}_{}.csv'
#           .format(year, month, day),
#           "w", newline='') as csvfile:
#     t_array = np.array(t_array, dtype=float64)
#     ACC_BVE_3DVAR = np.array(ACC_BVE_3DVAR, dtype=float64)
#     to_csv = np.column_stack((t_array, ACC_BVE_3DVAR))
#     np.savetxt(csvfile, to_csv, delimiter=",")


# # Save ACC_ERA_fc vs time
# with open('spherical_csv\\ACC\\ACC_ERA_fc_{}_{}_{}.csv'
#           .format(year, month, day),
#           "w", newline='') as csvfile:
#     t_array = np.array(t_array, dtype=float64)
#     ACC_ERA_fc = np.array(ACC_ERA_fc, dtype=float64)
#     to_csv = np.column_stack((t_array, ACC_ERA_fc))
#     np.savetxt(csvfile, to_csv, delimiter=",")


"""
Plot ACC
"""

fig = plt.figure(101, dpi=250)
for year_month in ['2019_07']:
    for day_i in [1, 10, 20]:
        with open('spherical_csv\\ACC\\ACC_BVE_{}_{}.csv'
                  .format(year_month, day_i),
                  "r", newline='') as csvfile:
            from_csv = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")
            t_array = from_csv[:, 0]
            ACC_BVE = from_csv[:, 1]

            p_BVE = plt.plot(
                t_array, ACC_BVE, color='r', lw=1,
                label='BVE')

for year_month in ['2019_07']:
    for day_i in [1, 10, 20]:
        with open('spherical_csv\\ACC\\ACC_BVE_3DVAR_{}_{}.csv'
                  .format(year_month, day_i),
                  "r", newline='') as csvfile:
            from_csv = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")
            t_array = from_csv[:, 0]
            ACC_BVE_3DVAR = from_csv[:, 1]

            p_3DVAR = plt.plot(t_array, ACC_BVE_3DVAR, color='g', lw=1,
                                label='BVE + 3DVAR')

for year_month in ['2019_07']:
    for day_i in [1, 10, 20]:
        with open('spherical_csv\\ACC\\ACC_ERA_fc_{}_{}.csv'
                  .format(year_month, day_i),
                  "r", newline='') as csvfile:
            from_csv = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")
            t_array = from_csv[:, 0]
            ACC_ERA_fc = from_csv[:, 1]

            p_ECMWF = plt.plot(t_array, ACC_ERA_fc, color='b', lw=1,
                                label='ECMWF')

left = 0
right = max(t_array)
plt.xlim(left, right)
plt.ylim(bottom=0, top=1.05)
plt.xlabel('Denbora (h)')
plt.ylabel('ACC')
plt.legend([Line2D([0], [0], color='r', lw=1),
            Line2D([0], [0], color='g', lw=1),
            Line2D([0], [0], color='b', lw=1)],
            ['BVE', 'BVE + 3DVAR', 'ECMWF'], loc='upper right')
plt.show()
