
"""
.. module:: validation_basic_BVE.py
    :synopsis: It contains the scripts used in the validation process of the 
    basic BVE model.
.. moduleauthor:: Kereman Alonso (alonso.kerman5@gmail.com)
"""

import numpy as np
from numpy import float64, arcsin
from sklearn.linear_model import LinearRegression

from matplotlib import pyplot as plt, cm, colors
from matplotlib.lines import Line2D
from mpl_toolkits.mplot3d import Axes3D

from netCDF4 import Dataset

import get_external_measurements as ext_measurement
import module_basic_BVE as sph


"""
Get the initial vorticity from ERA Interim.
"""
# omega: angular velocity of Earth in rad/s.
omega = 7.2921e-5
# a: radius of Earth in metres.
a = 6.3781e6

year = '2019'
month = '07'
# Get the information stored in the nc file
an = Dataset('..\\..\\data\\an_{}_{}.nc'.format(year, month), 'r')

# t0: index of initial time in ERA Interim analysis time array.
# t0 = 0 corresponds to the first day of a month at 00:00
day = 20
t0 = (day - 1) * 4
# pressure: index of pressure in ERA5 pressure array. pressure = 2
# corresponds to 500 hPa
pressure = 2

# order: order of antialiasing
order = 2
M = 63  # M = 479 gives highest possible mesh resolution

longit_dim = 3 * M + 1
latit_dim = (3 * M + 1) // 2 if (3 * M + 1) % 2 == 0 else\
    (3 * M + 1) // 2 + 1
longitudes, latitudes, weights =\
    sph.coordinates_and_weights(longit_dim, latit_dim)
latitudes_plot = -arcsin(latitudes)

# f: planetary vorticity
f = np.zeros(latit_dim, dtype=float64)
sph.get_planetary_vo(omega, latitudes, f)

vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)

ext_measurement.get_vorticity_matrix(an, t0, pressure, latitudes,
                                     longitudes, order, vorticity)

# Plot initial vorticity
# # Color map
# # # Normalize vorticity values
norm = cm.colors.Normalize(vmin=vorticity.min()*1e5, vmax=vorticity.max()*1e5)
fig = plt.figure(1, dpi=150)
color_plot = plt.pcolormesh(longitudes, latitudes_plot, vorticity*1e5,
                            cmap=cm.hot,
                            norm=norm)
fig.colorbar(color_plot)
fig.suptitle('Time: {}h'.format(0))
plt.show()

psi = np.zeros((latit_dim, longit_dim), dtype=float64)

"""
Plot and save BVE forecast.
"""
# # All time unis are in seconds.
# t_step = 900
# forecast_step = 6 * 3600
# max_time = 48 * 3600

# legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
# beta_matrix = np.zeros((M+2, M+2), dtype=float64)

# sph.associated_legendre(latitudes, legendre_matrix, beta_matrix)

# # Make forward and inverse transform of vorticity to get the vorticity
# # that can be represented with M modes
# t_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
# vo_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)
# sph.forward_transform(vorticity, weights, legendre_matrix, vo_coef)
# sph.inverse_transform(vo_coef, legendre_matrix, longitudes, t_vorticity)

# # # Plot truncated initial vorticity, and save
# sph.write_vo_csv(t_vorticity, '{}_{}'.format(year, month), t0 // 4 + 1, 0)
# norm = cm.colors.Normalize(vmin=vorticity.min()*1e5,
#                             vmax=vorticity.max()*1e5)
# fig = plt.figure(3, dpi=150)
# color_plot = plt.pcolormesh(longitudes, latitudes_plot, t_vorticity*1e5,
#                             cmap=cm.hot,
#                             norm=norm)
# fig.colorbar(color_plot)
# fig.suptitle(r'$t = {}h$'.format(0))
# plt.show()

# # Integrte forward in time
# sph.psi_forecast('{}_{}'.format(year, month), t0, t_step, forecast_step,
#                   max_time, omega, a,
#                   t_vorticity,
#                   norm, M)

"""
Plot ERA Interim analysis
"""
# t_range = 9  # how many sample times since t0

# ERA_an_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
# vo_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)

# legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
# beta_matrix = np.zeros((M+2, M+2), dtype=float64)
# sph.associated_legendre(latitudes, legendre_matrix, beta_matrix)

# for t in range(t_range):
#     ext_measurement.get_vorticity_matrix(an, t+t0, pressure, latitudes,
#                                           longitudes, order,
#                                           ERA_an_vorticity)

#     # Make forward and inverse transform of vorticity to get the vorticity
#     # that can be represented with M modes
#     sph.forward_transform(ERA_an_vorticity, weights,
#                           legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix,
#                           longitudes, ERA_an_vorticity)

#     # Plot vorticity
#     fig = plt.figure(50+t, dpi=150)
#     color_plot = plt.pcolormesh(longitudes, latitudes_plot,
#                                 ERA_an_vorticity*1e5,
#                                 cmap=cm.hot,
#                                 norm=norm)
#     fig.colorbar(color_plot)
#     fig.suptitle(r'$t = {}h$'.format(t*6))
#     plt.show()

"""
Vorticity distributions: BVE, ERA Interim forecast and analysis
"""
# t_range = 9  # how many sample times since t0

# BVE_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
# ERA_fc_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
# ERA_an_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
# vo_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)

# legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
# beta_matrix = np.zeros((M+2, M+2), dtype=float64)
# sph.associated_legendre(latitudes, legendre_matrix, beta_matrix)

# fc = Dataset('..\\..\\data\\fc_{}_{}.nc'.format(year, month), 'r')
# t_fc = 16  # 0  # 8  # 16


# for t in range(t_range):
#     # Get ERA Interim analysis and forecast vorticity matrices
#     ext_measurement.get_vorticity_matrix(an, t+t0, pressure, latitudes,
#                                           longitudes, order,
#                                           ERA_an_vorticity)
#     if (t == 0):
#         ext_measurement.get_vorticity_matrix(an, t0, pressure, latitudes,
#                                               longitudes, order,
#                                               ERA_fc_vorticity)
#     else:
#         ext_measurement.get_vorticity_matrix(fc, t-1+t_fc, pressure,
#                                               latitudes,
#                                               longitudes, order,
#                                               ERA_fc_vorticity)

#     # # Make forward and inverse transform of vorticity to get the
#     # # vorticity that can be represented with M modes
#     sph.forward_transform(ERA_an_vorticity, weights,
#                           legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix,
#                           longitudes, ERA_an_vorticity)

#     sph.forward_transform(ERA_fc_vorticity, weights,
#                           legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix,
#                           longitudes, ERA_fc_vorticity)

#     # Get BVE vorticity matrix
#     sph.read_vo_csv('{}_{}'.format(year, month), t0 // 4 + 1,
#                     t*6, BVE_vorticity)

#     # Plot distribution diagrams (x axis = fixed vorticity range)
#     bins_dim = 1000
#     hist_range = (-0.00015, 0.00015)
#     x = np.array([hist_range[0] + i * (hist_range[1] - hist_range[0])
#                   / bins_dim
#                   for i in range(bins_dim)], dtype=float64)
#     y_BVE = np.histogram(BVE_vorticity.flatten(), bins=bins_dim,
#                           range=hist_range)[0]
#     y_ERA_fc = np.histogram(ERA_fc_vorticity.flatten(), bins=bins_dim,
#                             range=hist_range)[0]
#     y_ERA_an = np.histogram(ERA_an_vorticity.flatten(), bins=bins_dim,
#                             range=hist_range)[0]

#     fig = plt.figure(500+t, dpi=150)
#     plt.xlim(hist_range[0]*1e5, hist_range[1]*1e5)
#     plt.ylim(bottom=0, top=160)
#     plt.plot(x*1e5, y_BVE, color='r', lw=1, label='BVE')
#     plt.plot(x*1e5, y_ERA_fc, color='b', lw=1, label='ECMWF')
#     plt.plot(x*1e5, y_ERA_an, color='g', lw=1, label='Analisia')

#     plt.title(r'$t = {}h$'.format(t*6))
#     plt.xlabel('Bortizitatea {}'.format(r'$(\times 10^5 s^{-1})$'))
#     plt.ylabel('Balio kopurua')
#     plt.legend()

#     plt.show()

#     # Plot distribution diagrams (x axis = relative vorticity range)
#     x = np.array([i / bins_dim * 100 for i in range(bins_dim)], dtype=float64)
#     y_BVE = np.histogram(BVE_vorticity.flatten(), bins=bins_dim)[0]
#     y_ERA_fc = np.histogram(ERA_fc_vorticity.flatten(), bins=bins_dim)[0]
#     y_ERA_an = np.histogram(ERA_an_vorticity.flatten(), bins=bins_dim)[0]

#     fig = plt.figure(600+t, dpi=150)
#     plt.xlim(0, 100)
#     plt.ylim(bottom=0, top=160)
#     plt.plot(x, y_BVE, color='r', lw=1, label='BVE')
#     plt.plot(x, y_ERA_fc, color='b', lw=1, label='ECMWF')
#     plt.plot(x, y_ERA_an, color='g', lw=1, label='Analisia')

#     plt.title(r'$t = {}h$'.format(t*6))
#     plt.xlabel('Bortizitatea balioen eremua (%)')
#     plt.ylabel('Balio kopurua')
#     plt.legend()

#     plt.show()


"""
Scatter plots: BVE vs ERA Iterim analysis,
                        ECMWF vs ERA Iterim analysis
"""

# t_range = 9  # how many sample times since t0

# BVE_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
# ERA_fc_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
# ERA_an_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
# vo_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)

# legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
# beta_matrix = np.zeros((M+2, M+2), dtype=float64)
# sph.associated_legendre(latitudes, legendre_matrix, beta_matrix)

# fc = Dataset('..\\..\\data\\fc_{}_{}.nc'.format(year, month), 'r')
# # t_fc is the time index of the 6h forecast vorticity in the fc.nc file.
# # Select t_fc = 0 for day 1 of the month, t_fc = 8 for day 10 and t_fc = 16
# # for day 20.
# t_fc = 16

# t_array = []
# RR_BVE = []
# a_BVE = []
# b_BVE = []

# R_BVE_err = []
# RR_BVE_err = []
# a_BVE_err = []
# b_BVE_err = []

# RR_ERA_fc = []
# a_ERA_fc = []
# b_ERA_fc = []

# R_ERA_err = []
# RR_ERA_err = []
# a_ERA_err = []
# b_ERA_err = []

# for t in range(t_range):
#     # Get ERA Interim analysis and forecast vorticity matrices
#     ext_measurement.get_vorticity_matrix(an, t+t0, pressure, latitudes,
#                                           longitudes, order,
#                                           ERA_an_vorticity)
#     if (t == 0):
#         ext_measurement.get_vorticity_matrix(an, t0, pressure, latitudes,
#                                               longitudes, order,
#                                               ERA_fc_vorticity)
#     else:
#         ext_measurement.get_vorticity_matrix(fc, t-1+t_fc, pressure,
#                                               latitudes,
#                                               longitudes, order,
#                                               ERA_fc_vorticity)

#     # # Make forward and inverse transform of vorticity to get the
#     # # vorticity that can be represented with M modes
#     sph.forward_transform(ERA_an_vorticity, weights,
#                           legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix,
#                           longitudes, ERA_an_vorticity)

#     sph.forward_transform(ERA_fc_vorticity, weights,
#                           legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix,
#                           longitudes, ERA_fc_vorticity)

#     # Get BVE vorticity matrix
#     sph.read_vo_csv('{}_{}'.format(year, month), t0 // 4 + 1,
#                     t*6, BVE_vorticity)

#     # Plot scatter plot and get Pearson correlation coefficient R
#     # # Flatten vorticity matrices
#     x = ERA_an_vorticity.flatten()
#     y_BVE = BVE_vorticity.flatten()
#     y_ERA_fc = ERA_fc_vorticity.flatten()

#     # # Get the values of R
#     R_BVE = np.corrcoef(x, y_BVE)[0, 1]
#     R_ERA_fc = np.corrcoef(x, y_ERA_fc)[0, 1]

#     t_array.append(t*6)
#     RR_BVE.append(R_BVE**2)
#     RR_ERA_fc.append(R_ERA_fc**2)

#     fig = plt.figure(50+t, dpi=150)
#     plt.axes().set_aspect('equal')
#     plt.scatter(x*1e5, y_BVE*1e5, s=0.2)
#     left = -0.00015*1e5
#     right = 0.00015*1e5
#     plt.xlim(left, right)
#     plt.ylim(left, right)
#     plt.title(r'$t = {}h$'.format(t*6))
#     plt.xlabel('Analisi bortizitatea {}'.format(r'$(\times 10^5 s^{-1})$'))
#     plt.ylabel('BVE bortizitatea {}'.format(r'$(\times 10^5 s^{-1})$'))

#     # # Store y = a + b * x linear regression parameters
#     x_ax = x.reshape((-1, 1))

#     regression_BVE = LinearRegression()
#     regression_BVE.fit(x_ax, y_BVE)
#     a_BVE.append(regression_BVE.intercept_)
#     b_BVE.append(regression_BVE.coef_)

#     # ==================================================== #
#     # --Compute BVE confidence intervals using bootstrap-- #

#     indices = np.array([i for i in range(x.shape[0])], dtype=int)
#     bootstrap_iterations = 1000
#     bootstrap_matrix = np.zeros((bootstrap_iterations, 4), dtype=float64)
#     for i in range(bootstrap_iterations):
#         samples = np.random.choice(indices, size=indices.shape[0],
#                                     replace=True)
#         samples_x = np.array([x[indice] for indice in samples], dtype=float64)
#         samples_y = np.array([y_BVE[indice] for indice in samples],
#                               dtype=float64)

#         bootstrap_matrix[i, 0] = np.corrcoef(samples_x, samples_y)[0, 1]
#         bootstrap_matrix[i, 1] = bootstrap_matrix[i, 0] ** 2

#         regression_samples = LinearRegression()
#         samples_x_ax = samples_x.reshape((-1, 1))
#         regression_samples.fit(samples_x_ax, samples_y)
#         bootstrap_matrix[i, 2] = regression_samples.intercept_
#         bootstrap_matrix[i, 3] = regression_samples.coef_

#     R_BVE_err.append((np.percentile(bootstrap_matrix[:, 0], 2.5),
#                       np.percentile(bootstrap_matrix[:, 0], 50),
#                       np.percentile(bootstrap_matrix[:, 0], 97.5)))

#     RR_BVE_err.append((np.percentile(bootstrap_matrix[:, 1], 2.5),
#                         np.percentile(bootstrap_matrix[:, 1], 50),
#                         np.percentile(bootstrap_matrix[:, 1], 97.5)))
    
#     a_BVE_err.append((np.percentile(bootstrap_matrix[:, 2], 2.5),
#                       np.percentile(bootstrap_matrix[:, 2], 50),
#                       np.percentile(bootstrap_matrix[:, 2], 97.5)))
    
#     b_BVE_err.append((np.percentile(bootstrap_matrix[:, 3], 2.5),
#                       np.percentile(bootstrap_matrix[:, 3], 50),
#                       np.percentile(bootstrap_matrix[:, 3], 97.5)))
    
#     # =============================================== #

#     regression_ERA_fc = LinearRegression()
#     regression_ERA_fc.fit(x_ax, y_ERA_fc)
#     a_ERA_fc.append(regression_ERA_fc.intercept_)
#     b_ERA_fc.append(regression_ERA_fc.coef_)

#     # ==================================================== #
#     # --Compute ERA confidence intervals using bootstrap-- #

#     indices = np.array([i for i in range(x.shape[0])], dtype=int)
#     bootstrap_iterations = 1000
#     bootstrap_matrix = np.zeros((bootstrap_iterations, 4), dtype=float64)
#     for i in range(bootstrap_iterations):
#         samples = np.random.choice(indices, size=indices.shape[0],
#                                     replace=True)
#         samples_x = np.array([x[indice] for indice in samples], dtype=float64)
#         samples_y = np.array([y_ERA_fc[indice] for indice in samples],
#                               dtype=float64)

#         bootstrap_matrix[i, 0] = np.corrcoef(samples_x, samples_y)[0, 1]
#         bootstrap_matrix[i, 1] = bootstrap_matrix[i, 0] ** 2

#         regression_samples = LinearRegression()
#         samples_x_ax = samples_x.reshape((-1, 1))
#         regression_samples.fit(samples_x_ax, samples_y)
#         bootstrap_matrix[i, 2] = regression_samples.intercept_
#         bootstrap_matrix[i, 3] = regression_samples.coef_

#     R_ERA_err.append((np.percentile(bootstrap_matrix[:, 0], 2.5),
#                       np.percentile(bootstrap_matrix[:, 0], 50),
#                       np.percentile(bootstrap_matrix[:, 0], 97.5)))

#     RR_ERA_err.append((np.percentile(bootstrap_matrix[:, 1], 2.5),
#                         np.percentile(bootstrap_matrix[:, 1], 50),
#                         np.percentile(bootstrap_matrix[:, 1], 97.5)))
    
#     a_ERA_err.append((np.percentile(bootstrap_matrix[:, 2], 2.5),
#                       np.percentile(bootstrap_matrix[:, 2], 50),
#                       np.percentile(bootstrap_matrix[:, 2], 97.5)))
    
#     b_ERA_err.append((np.percentile(bootstrap_matrix[:, 3], 2.5),
#                       np.percentile(bootstrap_matrix[:, 3], 50),
#                       np.percentile(bootstrap_matrix[:, 3], 97.5)))
    
#     # =============================================== #

#     plt.plot(x_ax[:, 0]*1e5, (a_BVE[-1] + b_BVE[-1] * x_ax[:, 0])*1e5, 'k',
#               linewidth=0.5)
#     plt.show()

# # Save R**2 BVE vs time
# with open('spherical_csv\\R_BVE_{}_{}_{}.csv'.format(year, month, day),
#           "w", newline='') as csvfile:
#     t_array = np.array(t_array, dtype=float64)
#     RR_BVE = np.array(RR_BVE, dtype=float64)
#     a_BVE = np.array(a_BVE, dtype=float64)
#     b_BVE = np.array(b_BVE, dtype=float64)
#     to_csv = np.column_stack((t_array, RR_BVE, a_BVE, b_BVE))
#     np.savetxt(csvfile, to_csv, delimiter=",")

# # Save R, R**2, a, b with errors (BVE)
# with open('spherical_csv\\R_BVE_err_{}_{}_{}.csv'.format(year, month, day),
#           "w", newline='') as csvfile:
#     t_array = np.array(t_array, dtype=float64)
#     R_BVE_err = np.array(R_BVE_err, dtype=(float64, 3))
#     RR_BVE_err = np.array(RR_BVE_err, dtype=(float64, 3))
#     a_BVE_err = np.array(a_BVE_err, dtype=(float64, 3))
#     b_BVE_err = np.array(b_BVE_err, dtype=(float64, 3))
#     to_csv = np.column_stack((t_array, R_BVE_err, RR_BVE_err,
#                               a_BVE_err, b_BVE_err))
#     np.savetxt(csvfile, to_csv, delimiter=",")

# # Save R**2 ERA fc vs time
# with open('spherical_csv\\R_ERA_fc_{}_{}_{}.csv'.format(year, month, day),
#           "w", newline='') as csvfile:
#     t_array = np.array(t_array, dtype=float64)
#     RR_ERA_fc = np.array(RR_ERA_fc, dtype=float64)
#     a_ERA_fc = np.array(a_ERA_fc, dtype=float64)
#     b_ERA_fc = np.array(b_ERA_fc, dtype=float64)
#     to_csv = np.column_stack((t_array, RR_ERA_fc, a_ERA_fc, b_ERA_fc))
#     np.savetxt(csvfile, to_csv, delimiter=",")

# # Save R, R**2, a, b with errors (BVE)
# with open('spherical_csv\\R_ERA_err_{}_{}_{}.csv'.format(year, month, day),
#           "w", newline='') as csvfile:
#     t_array = np.array(t_array, dtype=float64)
#     R_ERA_err = np.array(R_ERA_err, dtype=(float64, 3))
#     RR_ERA_err = np.array(RR_ERA_err, dtype=(float64, 3))
#     a_ERA_err = np.array(a_ERA_err, dtype=(float64, 3))
#     b_ERA_err = np.array(b_ERA_err, dtype=(float64, 3))
#     to_csv = np.column_stack((t_array, R_ERA_err, RR_ERA_err,
#                               a_ERA_err, b_ERA_err))
#     np.savetxt(csvfile, to_csv, delimiter=",")

"""
Northern hemisphere scatter plots: BVE vs ERA Iterim analysis,
                        ECMWF vs ERA Iterim analysis
"""

# t_range = 9  # how many sample times since 01/01/2019 00:00

# BVE_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
# ERA_fc_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
# ERA_an_vorticity = np.zeros((latit_dim, longit_dim), dtype=float64)
# vo_coef = np.zeros((M+1, 2*M+1), dtype=np.complex)

# legendre_matrix = np.zeros((M+2, 2*(M+1)+1, latit_dim), dtype=np.float64)
# beta_matrix = np.zeros((M+2, M+2), dtype=float64)
# sph.associated_legendre(latitudes, legendre_matrix, beta_matrix)

# fc = Dataset('..\\..\\data\\fc_{}_{}.nc'.format(year, month), 'r')
# t_fc = 16  # 0  # 8  # 16

# t_array = []
# RR_BVE = []
# a_BVE = []
# b_BVE = []

# R_BVE_err = []
# RR_BVE_err = []
# a_BVE_err = []
# b_BVE_err = []

# RR_ERA_fc = []
# a_ERA_fc = []
# b_ERA_fc = []

# for t in range(t_range):
#     # Get ERA Interim analysis and forecast vorticity matrices
#     ext_measurement.get_vorticity_matrix(an, t+t0, pressure, latitudes,
#                                           longitudes, order,
#                                           ERA_an_vorticity)
#     if (t == 0):
#         ext_measurement.get_vorticity_matrix(an, t0, pressure, latitudes,
#                                               longitudes, order,
#                                               ERA_fc_vorticity)
#     else:
#         ext_measurement.get_vorticity_matrix(fc, t-1+t_fc, pressure,
#                                               latitudes,
#                                               longitudes, order,
#                                               ERA_fc_vorticity)

#     # # Make forward and inverse transform of vorticity to get the
#     # # vorticity that can be represented with M modes
#     sph.forward_transform(ERA_an_vorticity, weights,
#                           legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix,
#                           longitudes, ERA_an_vorticity)

#     sph.forward_transform(ERA_fc_vorticity, weights,
#                           legendre_matrix, vo_coef)
#     sph.inverse_transform(vo_coef, legendre_matrix,
#                           longitudes, ERA_fc_vorticity)

#     # Get BVE vorticity matrix
#     sph.read_vo_csv('{}_{}'.format(year, month), t0 // 4 + 1,
#                     t*6, BVE_vorticity)

#     # Plot scatter plot and get Pearson correlation coefficient R
#     # # Flatten vorticity matrices
#     x = ERA_an_vorticity.flatten()[:latit_dim*longit_dim//2]
#     y_BVE = BVE_vorticity.flatten()[:latit_dim*longit_dim//2]
#     y_ERA_fc = ERA_fc_vorticity.flatten()[:latit_dim*longit_dim//2]

#     # # Get the values of R
#     R_BVE = np.corrcoef(x, y_BVE)[0, 1]
#     R_ERA_fc = np.corrcoef(x, y_ERA_fc)[0, 1]

#     t_array.append(t*6)
#     RR_BVE.append(R_BVE**2)
#     RR_ERA_fc.append(R_ERA_fc**2)

#     fig = plt.figure(50+t, dpi=150)
#     plt.scatter(x, y_BVE, s=0.2)
#     left = -0.00015
#     right = 0.00015
#     plt.xlim(left, right)
#     plt.ylim(left, right)
#     plt.title(r'$t = {}h$'.format(t*6))
#     plt.xlabel('Analisi bortizitatea {}'.format(r'$(s^{-1})$'))
#     plt.ylabel('BVE bortizitatea {}'.format(r'$(s^{-1})$'))

#     # # Store y = a + b * x linear regression parameters
#     x_ax = x.reshape((-1, 1))

#     regression_BVE = LinearRegression()
#     regression_BVE.fit(x_ax, y_BVE)
#     a_BVE.append(regression_BVE.intercept_)
#     b_BVE.append(regression_BVE.coef_)

#     regression_ERA_fc = LinearRegression()
#     regression_ERA_fc.fit(x_ax, y_ERA_fc)
#     a_ERA_fc.append(regression_ERA_fc.intercept_)
#     b_ERA_fc.append(regression_ERA_fc.coef_)

#     plt.plot(x_ax[:, 0], a_BVE[-1] + b_BVE[-1] * x_ax[:, 0], 'k',
#               linewidth=0.5)
#     plt.show()

# # Save R**2 BVE vs time
# with open('spherical_csv\\northern_hemisphere\\R_BVE_{}_{}_{}.csv'.format(
#         year, month, day),
#           "w", newline='') as csvfile:
#     t_array = np.array(t_array, dtype=float64)
#     RR_BVE = np.array(RR_BVE, dtype=float64)
#     a_BVE = np.array(a_BVE, dtype=float64)
#     b_BVE = np.array(b_BVE, dtype=float64)
#     to_csv = np.column_stack((t_array, RR_BVE, a_BVE, b_BVE))
#     np.savetxt(csvfile, to_csv, delimiter=",")

# # Save R**2 ERA fc vs time
# with open('spherical_csv\\northern_hemisphere\\R_ERA_fc_{}_{}_{}.csv'.format(
#         year, month, day),
#           "w", newline='') as csvfile:
#     t_array = np.array(t_array, dtype=float64)
#     RR_ERA_fc = np.array(RR_ERA_fc, dtype=float64)
#     a_ERA_fc = np.array(a_ERA_fc, dtype=float64)
#     b_ERA_fc = np.array(b_ERA_fc, dtype=float64)
#     to_csv = np.column_stack((t_array, RR_ERA_fc, a_ERA_fc, b_ERA_fc))
#     np.savetxt(csvfile, to_csv, delimiter=",")

"""
R**2 vs time plots: BVE winter and summer plot, BVE and ECMWF plot
"""

# # BVE winter and summer plot
# fig = plt.figure(100, dpi=250)

# for year_month in ['2018_12', '2019_01']:
#     for day_i in [1, 10, 20]:
#         with open('spherical_csv\\northern_hemisphere\\R_BVE_{}_{}.csv'.format(year_month, day_i),
#                   "r", newline='') as csvfile:
#             from_csv = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")
#             t_array = from_csv[:, 0]
#             RR_array = from_csv[:, 1]

#             plt.plot(t_array, RR_array, color='deepskyblue', lw=1,
#                      label='Negua')

# for year_month in ['2019_07', '2019_08']:
#     for day_i in [1, 10, 20]:
#         with open('spherical_csv\\northern_hemisphere\\R_BVE_{}_{}.csv'.format(year_month, day_i),
#                   "r", newline='') as csvfile:
#             from_csv = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")
#             t_array = from_csv[:, 0]
#             RR_array = from_csv[:, 1]

#             plt.plot(t_array, RR_array, color='orange', lw=1, label='Uda')

# left = 0
# right = max(t_array)
# plt.xlim(left, right)
# plt.ylim(bottom=0, top=1.05)
# plt.xlabel('Denbora (h)')
# plt.ylabel(r'$R^2$')
# plt.legend([Line2D([0], [0], color='deepskyblue', lw=1),
#             Line2D([0], [0], color='orange', lw=1)], ['Negua', 'Uda'])
# plt.show()

# # ECMWF winter and summer plot
# fig = plt.figure(100, dpi=250)

# for year_month in ['2018_12', '2019_01']:
#     for day_i in [1, 10, 20]:
#         with open('spherical_csv\\northern_hemisphere\\R_ERA_fc_{}_{}.csv'.format(year_month, day_i),
#                   "r", newline='') as csvfile:
#             from_csv = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")
#             t_array = from_csv[:, 0]
#             RR_array = from_csv[:, 1]

#             plt.plot(t_array, RR_array, color='deepskyblue', lw=1,
#                      label='Negua')

# for year_month in ['2019_07', '2019_08']:
#     for day_i in [1, 10, 20]:
#         with open('spherical_csv\\northern_hemisphere\\R_ERA_fc_{}_{}.csv'.format(year_month, day_i),
#                   "r", newline='') as csvfile:
#             from_csv = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")
#             t_array = from_csv[:, 0]
#             RR_array = from_csv[:, 1]

#             plt.plot(t_array, RR_array, color='orange', lw=1, label='Uda')

# left = 0
# right = max(t_array)
# plt.xlim(left, right)
# plt.ylim(bottom=0, top=1.05)
# plt.xlabel('Denbora (h)')
# plt.ylabel(r'$R^2$')
# plt.legend([Line2D([0], [0], color='deepskyblue', lw=1),
#             Line2D([0], [0], color='orange', lw=1)], ['Negua', 'Uda'])
# plt.show()

# # BVE and ECMWF plot
# fig = plt.figure(101, dpi=250)
# for year_month in ['2018_12', '2019_01', '2019_07', '2019_08']:
#     for day_i in [1, 10, 20]:
#         with open('spherical_csv\\R_BVE_{}_{}.csv'.format(year_month, day_i),
#                   "r", newline='') as csvfile:
#             from_csv = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")
#             t_array = from_csv[:, 0]
#             RR_array = from_csv[:, 1]

#             p_BVE = plt.plot(
#                 t_array, RR_array, color='r', lw=1,
#                 label='BVE')

# for year_month in ['2018_12', '2019_01', '2019_07', '2019_08']:
#     for day_i in [1, 10, 20]:
#         with open('spherical_csv\\R_ERA_fc_{}_{}.csv'.format(year_month,
#                                                               day_i),
#                   "r", newline='') as csvfile:
#             from_csv = np.loadtxt(csvfile, dtype=np.float64, delimiter=",")
#             t_array = from_csv[:, 0]
#             RR_array = from_csv[:, 1]

#             p_ECMWF = plt.plot(t_array, RR_array, color='b', lw=1,
#                                 label='ECMWF')

# left = 0
# right = max(t_array)
# plt.xlim(left, right)
# plt.ylim(bottom=0, top=1.05)
# plt.xlabel('Denbora (h)')
# plt.ylabel(r'$R^2$')
# plt.legend([Line2D([0], [0], color='r', lw=1),
#             Line2D([0], [0], color='b', lw=1)], ['BVE', 'ECMWF'])
# plt.show()
